#!/bin/sh
#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2015, 2016. All Rights Reserved.
#
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************

########################################################
#
# Script : CoreAutomationPrep.sh
# Purpose :
# - Use ${localTags} To drive "Link Points"
# - Get credentials files
#
########################################################

scriptName=$0
envProp=/etc/env.properties
chmod 755 $envProp

CurrentWorkPath=`pwd -P`
echo AUTO_PREP=$CurrentWorkPath >> $envProp

. $envProp


################
##  FUNCTIONS  #
################

checkAndCreateLinkFolder()
{
  tempLinkDir=$1
  tempLink=$2
  tempTargetDir=$3

  if [ -L ${tempLink} ];then
    if [ "$(readlink "${tempLink}")" == ${tempTargetDir} ]; then
      echo "link [${tempLink}] to the folder [${tempTargetDir}] already exists"
      result=$(getFolderPermission ${tempTargetDir})
      if [ $? == 0 ]; then
      	 if [ "$result" == "777" ];then
                echo "[${tempTargetDir}] has correct permission: [$result]"
         else
                echo "ERROR: Wrong permission set for [${tempTargetDir}], current setting is [$result]"
                exit 1
         fi
	  else
         echo "ERROR: Couldn't get the permission for directory [${tempTargetDir}]"
         exit 1
      fi	   
    else
      echo "ERROR: script "${scriptName}" Link [${tempLink}] exists but not to folder [${tempTargetDir}]"
      exit 1
    fi 
  else
    echo "link from [${tempLink}] to [${tempTargetDir}]"
    mkdir -p ${tempTargetDir}
    ln -s ${tempTargetDir} ${tempLink}
    chmod 777 ${tempTargetDir}
    # check link created
    if [ -L ${tempLink} ] && [ "$(readlink "${tempLink}")" == ${tempTargetDir} ]; then
      echo "link created"
    else
      echo "ERROR: script "${scriptName}" Failed to create link"
      exit 1
    fi
  fi
  
  echo "list the folder with the links ["${tempLinkDir}"]"
  ls -la ${tempLinkDir}
  
  echo "list the target folder ["${tempTargetDir}"]"
  ls -la ${tempTargetDir}

}


checkAndCreateFolder()
{

  tempFolder=$1
  echo "check ["${tempFolder}"] exists"
  if [ -d $tempFolder ];then
    echo "folder exists"
  else
    echo "folder does not exist, create it..."
    mkdir -p ${tempFolder}
    if [ -d $tempFolder ];then
      echo "folder created"
    else
      echo "ERROR: script "${scriptName}" Failed to create folder"
      exit 1
    fi
  fi

}


moveFile()
{

  tempFile=$1
  tempFolder=$2
  
  echo "move [${tempFile}] to [${tempFolder}/${tempFile}]"
  if [ -f ${tempFile} ]; then
    echo "File  exists"
  else
    echo "ERROR: script "${scriptName}" File does not exist"
    exit 1
  fi
  
  if [ -d ${tempFolder} ]; then
    echo "Folder  exists"
  else
    echo "ERROR: script "${scriptName}" Folder does not exist"
    exit 1
  fi
  
  mv ${tempFile} ${tempFolder}
  if [ -f ${tempFolder}/${tempFile} ]; then
    echo "File moved"
  else
    echo "ERROR: script "${scriptName}" File not moved"
    exit 1
  fi

}


## usage checkFileExists tempFile
checkFileExists()
{
  tempFile=$1
  if [ -f $tempFile ]; then
    echo "file [$tempFile] exists"
  else
    echo "ERROR: script "${scriptName}" [$tempFile] does not exist"
    exit 1
  fi
}


# will check the package is installed
checkRHPackageInstalled()
{
  package=$1
  echo "---checking ${package} installed---"
  rpm -q ${package}
  if [ $? != 0 ]
  then
    echo "${package} package not installed"
    rhPackageInstalled="FALSE"
  else
    echo "${package} package is installed."
    rhPackageInstalled="TRUE"
  fi
  
}


# will check command in PATH using which
checkCommandInPath()
{
  command=$1
  echo "---checking ${command} in PATH---"
  which ${command}
  if [ $? != 0 ]
  then
    echo "${command} command not in PATH"
    rhCommandInPath="FALSE"
  else
    echo "${command} command in PATH."
    rhCommandInPath="TRUE"
  fi
  
}


## usage checkLibFileExists tempFile
checkLibFileExists()
{
  tempFile=$1
  echo "---checking lib ${tempFile} exists---"
  if [ -f $tempFile ]; then
    echo "lib file [$tempFile] exists"
    rhLibFileExists="TRUE"
  else
    echo "lib file [$tempFile] does not exists"
    rhLibFileExists="FALSE"
  fi
}

getFolderPermission()
{
	folderName=$1
    retval=`stat -c "%a" $folderName`
    if [ $? == 0 ];then
    	echo "$retval"
    else
        echo "ERROR: Failed to get folder permission for folderName [$folderName]"
        exit 1
    fi
}


## calculate the application release eg 60, 601, 602
setAppRelease()
{
  # extract the prefix and then the release
  appMediaContent=`echo ${appMediaRoot} | cut -c -2`
  if [ "SS" == $appMediaContent ]; then
    # SSE
    appRelease=`echo ${appMediaRoot} | cut -c 4-`
  elif  [ "CL" == $appMediaContent ]; then
    # CLM
    appRelease=`echo ${appMediaRoot} | cut -c 4-`
  elif  [ "CE" == $appMediaContent ]; then
    appRelease=`echo ${appMediaRoot} | cut -c 3-`
  else
    echo "-- ERROR [${scriptName}] : unsupported prefix ["${appMediaContent}"] in ["${appMediaRoot}"]"
    exit 1
  fi
  # allow for 60 and 600 being the same stream
  if [ "60" == $appRelease ]; then
    echo "modifying appRelease [${appRelease}] to become [600]"
    appRelease=600
  fi
  echo "appMediaRoot [${appMediaRoot}], appMediaContent [${appMediaContent}], appRelease [${appRelease}]"

}


## calculate the iFix stream and patch
## - eg 601 & 001 from CLM_601_iFix001
## - eg 601 & 1 from DM_601_iFix_1
## - eg 601 & 1 from Rhapsody_DM_ModelServer_601_iFix_1
##
## returns values in $tempReturnStream & $tempReturnPatch
setIfixStreamPatch()
{
  tempApp=$1
  tempVersion=$2
  
  if [ "CLM" == $tempVersion ] || [ "DM" == $tempVersion ] || [ "DMMS" == $tempVersion ]; then
    echo "[$tempVersion] iFix being installed via the pipeline"
    tempReturnStream=${appRelease}
    tempReturnPatch="pipeline"
  else
    echo "extract the iFix stream & patch strings"
    if [ "CLM" == $tempApp ]; then
      tempReturnStream=`echo ${tempVersion} | cut -d"_" -f 2`      
      tempReturnPatch=`echo ${tempVersion} | rev | cut -d"x" -f 1 | rev`      
    elif [ "DM" == $tempApp ]; then
      tempReturnStream=`echo ${tempVersion} | cut -d"_" -f 2`
      tempReturnPatch=`echo ${tempVersion} | rev | cut -d"_" -f 1 | rev`
    elif [ "DMMS" == $tempApp ]; then
      tempReturnStream=`echo ${tempVersion} | cut -d"_" -f 4`
      tempReturnPatch=`echo ${tempVersion} | rev | cut -d"_" -f 1 | rev`
    else
      echo "-- ERROR [${scriptName}] : do not support iFix product [${tempApp}]"
      exit 1
    fi
  fi
  # allow for 60 and 600 being the same stream
  if [ "60" == $tempReturnStream ]; then
    echo "modifying iFix Release [${tempReturnStream}] to become [600]"
    tempReturnStream=600
  fi
  
  if [ "pipeline" != $tempReturnPatch ]; then
    # remove last char if it is not a "numeric char [0-9]"  
    tempLastChar=`echo ${tempReturnPatch: -1}`
    echo "string [${tempReturnPatch}] last char [$tempLastChar]"
    case $tempLastChar in
      [0-9])
            echo "char is numeric, so leave as is"
            ;;
          *)
            echo "char is NOT numeric, so remove it!"
            tempStrLen=${#tempReturnPatch}
            tempStrLen=$((tempStrLen - 1))
            tempReturnPatch=`echo ${tempReturnPatch:0:$tempStrLen}`
    esac
  
    # pack the [$tempReturnPatch] so that it is always 3 characters
    echo "modifying iFix Patch [${tempReturnPatch}] to ensure it has 3 characters"
    tempReturnPatch=`echo "000"${tempReturnPatch} | rev | cut -c 1-3 | rev`
  fi
  
  echo "iFix-Version [${tempVersion}], iFix-Stream [${tempReturnStream}] and iFix-Patch [$tempReturnPatch]"
}



###########
##  MAIN  #
###########


# ucdRootDir=/IBMQD
# take value from envvironment
ucdRootDir=${QD_Root}
finalCredentialsDir=/tmp/IM_Keyring

####################################
# ensure umask setting is correct level
# The user created file should be readable and executable by everyone
####################################
umaskDirectoryBits=`umask | cut -c1`
umaskUserBits=`umask | cut -c2`
umaskGroupBits=`umask | cut -c3`
umaskOtherBits=`umask | cut -c4`
if [ $umaskDirectoryBits == 0 ];then
	echo "System has correct umask setting for the directory creation permission: [$umaskDirectoryBits]"
else
	echo "ERROR: Please set correct umask settings for directory bit the value must be 0, current setting is [$umaskUserBits]"
	exit 1
fi
if [ $umaskUserBits -lt 2 ];then
	echo "System has correct umask settings for Owner: [$umaskUserBits]"
else
	echo "ERROR: Please set correct umask settings for Owner, the value must be 0 or 1, current setting is [$umaskUserBits]"
	exit 1
fi

if [ $umaskGroupBits -lt 3 ];then
	echo "System has correct umask settings for Group: [$umaskGroupBits]"
else
	echo "ERROR: Please set correct umask settings for Group, the value must be 0, 1 or 2, current setting is [$umaskUserBits]"
	exit 1
fi

if [ $umaskGroupBits -lt 3 ];then
	echo "System has correct umask settings for Group: [$umaskOtherBits]"
else
	echo "ERROR: Please set correct umask settings for Other, the value must be 0, 1 or 2, current setting is [$umaskOtherBits]"
	exit 1
fi

####################################
# ensure the UCD ROOT DIR exists
checkAndCreateFolder ${ucdRootDir}
result=$(getFolderPermission ${ucdRootDir})
if [ $? == 0 ]; then
        if [ "$result" -ge "775" ];then
                echo "${ucdRootDir} has correct permission: [$result]"
        else
                echo "ERROR: Require 775 or 777 permission set for [${ucdRootDir}], current setting is [$result]"
                exit 1
        fi
else
        echo "ERROR: Couldn't get the permission for directory [${ucdRootDir}]"
        exit 1
fi

####################################
# Check /home directory permission
####################################
result=$(getFolderPermission /home)
if [ $? == 0 ]; then
        if [ "$result" == "755" ] || [ "$result" == "777" ] || [ "$result" == "757" ] || [ "$result" == "775" ];then
                echo "/home has correct permission: [$result]"
        else
                echo "ERROR: wrong permission set for /home, current setting is [$result]"
                exit 1
        fi
else
        echo "ERROR: Couldn't get the permission for directory /home"
        exit 1
fi

##################################
# Ensure domain name existed
# SSO configuration requires to have the domain name
##################################
domainName=`hostname -d`
if [ $? == 0 ]; then
	if [ "X"${domainName}"X" == "XX" ];then
		echo "ERROR: The domain name can not be empty"
		exit 1
	else
		echo "Domain Name is [$domainName]"
	fi
else
	echo "ERROR: Failed to retrieve the domain name using 'hostname -d' command, please fix it."
	exit 1
fi


####################################
# may need to check for RH packages 
checkPreReqXvfbFlag=FALSE
checkPreReqDMlibsFlag=FALSE
packagesMissing=FALSE

####################################
# may need to check OS 
bRH6flag=FALSE
bRH7flag=FALSE

RHversion=`cat /etc/redhat-release`
echo "Current Red Hat Release $RHversion"

echo $RHversion | grep "Maipo"
if [ $? -eq 0 ]; then
  echo "OS is Red Hat 7"
  bRH7flag=TRUE
fi
echo $RHversion | grep "Santiago"
if [ $? -eq 0 ]; then
  echo "OS is Red Hat 6"
  bRH6flag=TRUE
fi

##if [ "TRUE" == $bRH6flag ] || [ "TRUE" == $bRH7flag ]; then
##  echo "supported OS"
##else
##  echo "ERROR: not a supported OS"
##  exit 1
##fi


####################################
# ensure that the links are in place 
# to keep the rest of the install scripts working
echo "Ensure that the links are in place...."
echo "localTagList ["${localTags}"]"
tagList=${localTags//,/ }
for tag in $tagList
do
  echo "process tag ["${tag}"]"
  if [[ $tag == "IHS" ]] || [[ $tag == "WAS" ]]; then
   linkDirs="WebSphere,HTTPServer,Plugins,IMShared,JazzTeamServer,InstallationManager"
   linkList=${linkDirs//,/ }
   for link in $linkList
   do
     theLinkDir=/opt/IBM
     # link will appear in ${theLinkDir} folder, so it must exis
     # We do not need check permission of /opt/IBM, as the link will run by root
     checkAndCreateFolder ${theLinkDir}
     theLink=${theLinkDir}/${link}
     theTarget=${ucdRootDir}/${link}
     # create the link
     # We already check the the Target dir inside function as all target should have 777 permission
     checkAndCreateLinkFolder ${theLinkDir} ${theLink} ${theTarget}
     
   done
  fi
  
  if [[ $tag == "DB" && "$DB_VENDOR" == "db2" ]]; then
     # link [/db2inst1] will appear in "/" folder
     theLinkDir=/db2inst1
     theLink=${theLinkDir}
     theTarget=${ucdRootDir}/database
     # create the link
     checkAndCreateLinkFolder ${theLinkDir} ${theLink} ${theTarget}
     
     # need a second link [/db2fs] also in "/" folder
     theLinkDir=/db2fs
     theLink=${theLinkDir}
     theTarget=/db2inst1
     # create the link
     checkAndCreateLinkFolder ${theLinkDir} ${theLink} ${theTarget}
     
     # need a folder to hold CLM data warehouse folder
     checkAndCreateFolder ${ucdRootDir}/DWFolder
     chmod 777  ${ucdRootDir}/DWFolder
  fi
  
  if [[ $tag == "DB" && "$DB_VENDOR" == "oracle" ]]; then
     # need a folder to hold oracle database installation
     checkAndCreateFolder ${ucdRootDir}/database
     chmod 777 ${ucdRootDir}/database
     # need a folder to hold CLM data warehouse folder
     checkAndCreateFolder ${ucdRootDir}/DWFolder
     chmod 777  ${ucdRootDir}/DWFolder
  fi

  # determine which repReqs to check
  if [[ $tag == "RM" ]]; then
    checkPreReqXvfbFlag=TRUE
  fi
  
  if [[ $tag == "DM" ]]; then
    checkPreReqXvfbFlag=TRUE
    checkPreReqDMlibsFlag=TRUE
  fi
  
done


#############################################################################################################
# Check if iFix is enabled that the iFix versions and base version match
# rule1. a CLM iFix must be present to install a DM or DMMS iFix eg being installed as a pair or triple
# rule2. iFix must be on the same stream as the build eg 601 iFix only installed along with a CE601 build
# rule3: DMY or DMMSY can only be installed alongside CLM iFix00Y (that is all that DM test team test)
# rule4. DM or DMMS iFixes can be installed independently of each other

echo "--- check iFix rules ---"
echo "appMediaRoot [$appMediaRoot]"
echo "iFix enabled [$iFixEnabled]"
echo "CLMiFixVersion [$CLMiFixVersion]"
echo "DMiFixVersion [$DMiFixVersion]"
echo "DMMSiFixVersion [$DMMSiFixVersion]"

# validate the rules
if [ "FALSE" == ${iFixEnabled} ]; then
  ## do nothing
  echo "not installing iFix as $!iFixEnabled [${iFixEnabled}], so not checking iFix version compatability"
else
  echo "potentially installing iFix as $!iFixEnabled [${iFixEnabled}]"

  CLMiFixSettingsValid=TRUE
  DMiFixSettingsValid=TRUE
  DMMSiFixSettingsValid=TRUE

  #rule1. a CLM iFix must be present to install a DM or DMMS iFix eg being installed as a pair or triple
  if [ "NONE" == $CLMiFixVersion ]; then  
    if [ "NONE" == $DMiFixVersion ] && [ "NONE" == $DMMSiFixVersion ]; then
      echo "no iFixes are being installed as $!CLMiFixVersion [${CLMiFixVersion}], $!DMiFixVersion [${DMiFixVersion}] and $!DMMSiFixVersion [${DMMSiFixVersion}]"
    else
      echo "-- ERROR [${scriptName}] : can not install $!DMiFixVersion [${DMiFixVersion}] or $!DMMSiFixVersion [${DMMSiFixVersion}] as $!CLMiFixVersion [${CLMiFixVersion}]"
      echo "-- $!CLMiFixVersion must be set as well!"
      CLMiFixSettingsValid=FALSE
      DMiFixSettingsValid=FALSE
      DMMSiFixSettingsValid=FALSE
    fi
  else
    ## test for valid selections
    
    ## calculate the application release ${appRelease} eg 60, 600, 601, 602 eg 601 from CE601
    setAppRelease

    #rule2. iFix must be on the same stream as the build eg 601 iFix only installed along with a CE601 build
    if [ "NONE" == $CLMiFixVersion ]; then
      echo "although enabled no CLM iFix is being installed as $!CLMiFixVersion [${CLMiFixVersion}], so not checking CLM iFix version compatability"
    else
      echo "now checking CLM iFix version compatability..."
      # iFix version and base version have to match
      setIfixStreamPatch "CLM" $CLMiFixVersion
      #rule2. iFix must be on the same stream as the build eg 601 iFix only installed along with a CE601 build
      ## only continue if ${tempReturnStream} matches ${appRelease}
      if [ ${tempReturnStream} != ${appRelease} ]; then
        echo "-- ERROR [${scriptName}] : not installing CLM iFix as iFix-Stream [${tempReturnStream}] does not match appRelease [${appRelease}]"
        CLMiFixSettingsValid=FALSE
      else
        echo "potentially installing CLM iFix as iFix-Stream [${tempReturnStream}] matches appRelease [${appRelease}]"
      fi
      CLMiFixPatch=$tempReturnPatch
    fi
  
    #rule4. DM or DMMS iFixes can be installed independently of each other
    if [ "NONE" == $DMiFixVersion ]; then
      echo "although enabled no DM iFix is being installed as $!DMiFixVersion [${DMiFixVersion}], so not checking DM iFix version compatability"
    else
      echo "now checking DM iFix version compatability..."
      # iFix version and base version have to match
      setIfixStreamPatch "DM" $DMiFixVersion
      #rule2. iFix must be on the same stream as the build eg 601 iFix only installed along with a CE601 build
      ## continue if ${tempReturnStream} matches ${appRelease}
      if [ ${tempReturnStream} != ${appRelease} ]; then
        echo "-- ERROR [${scriptName}] : not installing DM iFix as iFix-Stream [${tempReturnStream}] does not match appRelease [${appRelease}]"
        DMiFixSettingsValid=FALSE
      else
        echo "potentially installing DM iFix as iFix-Stream [${tempReturnStream}] matches appRelease [${appRelease}]"
        #rule3: DM iFixY can only be installed alongside CLM iFix00Y (that is all that DM test team test)
        ## finally only continue if ${tempReturnPatch} matches ${CLMiFixPatch}
        if [ ${tempReturnPatch} != ${CLMiFixPatch} ]; then
          echo "-- ERROR [${scriptName}] : not installing DM iFix as DM-Patch [${tempReturnPatch}] does not match CLM-Patch [${CLMiFixPatch}]"
          DMiFixSettingsValid=FALSE
        else
          echo "potentially installing DM iFix as DM-Patch [${tempReturnPatch}] matches CLM-Patch [${CLMiFixPatch}]"
        fi
      fi
    fi
    
    #rule4. DM or DMMS iFixes can be installed independently of each other
    if [ "NONE" == $DMMSiFixVersion ]; then
      echo "although enabled no DMMS iFix is being installed as $!DMMSiFixVersion [${DMMSiFixVersion}], so not checking DMMS iFix version compatability"
    else
      echo "now checking DMMS iFix version compatability..."
      # iFix version and base version have to match
      setIfixStreamPatch "DMMS" $DMMSiFixVersion
      #rule2. iFix must be on the same stream as the build eg 601 iFix only installed along with a CE601 build
      ## only continue if ${tempReturnStream} matches ${appRelease}
      if [ ${tempReturnStream} != ${appRelease} ]; then
        echo "-- ERROR [${scriptName}] : not installing DMMS iFix as iFix-Stream [${tempReturnStream}] does not match appRelease [${appRelease}]"
        DMMSiFixSettingsValid=FALSE
      else
        echo "potentially installing DMMS iFix as iFix-Stream [${tempReturnStream}] matches appRelease [${appRelease}]"
        #rule3: DMMS iFixY can only be installed alongside CLM iFix00Y (that is all that DM test team test)
        ## finally only continue if ${tempReturnPatch} matches ${CLMiFixPatch}
        if [ ${tempReturnPatch} != ${CLMiFixPatch} ]; then
          echo "-- ERROR [${scriptName}] : not installing DMMS iFix as DMMS-Patch [${tempReturnPatch}] does not match CLM-Patch [${CLMiFixPatch}]"
          DMMSiFixSettingsValid=FALSE
        else
          echo "potentially installing DMMS iFix as DMMS-Patch [${tempReturnPatch}] matches CLM-Patch [${CLMiFixPatch}]"
        fi
      fi
    fi
  fi
  
  echo "$!CLMiFixSettingsValid [$CLMiFixSettingsValid], $!DMiFixSettingsValid [$DMiFixSettingsValid], $!DMMSiFixSettingsValid [$DMMSiFixSettingsValid]"
  if [ "TRUE" == $CLMiFixSettingsValid ] && [ "TRUE" == $DMiFixSettingsValid ] && [ "TRUE" == $DMMSiFixSettingsValid ]; then
    echo "checks finished, potentially installing iFixes"
  else
    echo "ERROR: one or more checks failed, not installing iFixes"
    exit 1
  fi  
  
fi


########################################
# Check RH preReq packages are installed
echo "--- check RH preReq packages are installed ---"

# -- check expect package for SSH is installed and in the PATH
## ===========================================================
package=expect
checkRHPackageInstalled ${package}
if [ "FALSE" == $rhPackageInstalled ]; then 
  packagesMissing="TRUE"
fi
checkCommandInPath ${package}
if [ "FALSE" == $rhCommandInPath ]; then 
  packagesMissing="TRUE"
fi


# -- check Xvfb package for RM & DM pictures is installed and in the PATH
## ======================================================================
if [ "TRUE" == $checkPreReqXvfbFlag ]; then 
  package=xorg-x11-server-Xvfb
  checkRHPackageInstalled ${package}
  if [ "FALSE" == $rhPackageInstalled ]; then 
    packagesMissing="TRUE"
  fi
  packageCommand=Xvfb
  checkCommandInPath ${packageCommand}
  if [ "FALSE" == $rhCommandInPath ]; then 
    packagesMissing="TRUE"
  fi
fi

# -- check DM libraries present
# -- Note: checking for preReqs packages for DM server and desktop products
## ========================================================================
if [ "TRUE" == $checkPreReqDMlibsFlag ]; then
  
  # -- (RH6 and RH7) check 32 bit libXdmcp.so.6 package for DM 
  ## =======================================================
  package=libXdmcp.i686
  checkRHPackageInstalled ${package}
  if [ "FALSE" == $rhPackageInstalled ]; then
    packagesMissing="TRUE"
  fi
  libFile=/usr/lib/libXdmcp.so.6
  checkLibFileExists ${libFile}
  if [ "FALSE" == $rhLibFileExists ]; then
    packagesMissing="TRUE"
  fi
  
  # -- (RH6 and RH7) check 32 bit libXtst.i686 package for DM 
  ## =======================================================
  package=libXtst.i686
  checkRHPackageInstalled ${package}
  if [ "FALSE" == $rhPackageInstalled ]; then
    packagesMissing="TRUE"
  fi
  
  # -- (RH6 and RH7) check 32 bit libcanberra-gtk2.i686 package for DM 
  ## =======================================================
  package=libcanberra-gtk2.i686
  checkRHPackageInstalled ${package}
  if [ "FALSE" == $rhPackageInstalled ]; then 
    packagesMissing="TRUE"
  fi
  
  # -- (RH6 and RH7) check 32 bit libXmu.i686 package for DM 
  ## =======================================================
  package=libXmu.i686
  checkRHPackageInstalled ${package}
  if [ "FALSE" == $rhPackageInstalled ]; then 
    packagesMissing="TRUE"
  fi
  
  # -- (RH6 and RH7) check 32 bit gtk2.i686 package for DM 
  ## =======================================================
  package=gtk2.i686
  checkRHPackageInstalled ${package}
  if [ "FALSE" == $rhPackageInstalled ]; then 
    packagesMissing="TRUE"
  fi


  if [ "TRUE" == $bRH6flag ]; then
    echo "+++ check Red Hat 6 specific packages"
    
    # -- (RH6 only) check 32 bit PackageKit-gtk3-module.i686 package for DM (may not be needed for server)
    ## =======================================================
    package=PackageKit-gtk-module.i686
    checkRHPackageInstalled ${package}
    if [ "FALSE" == $rhPackageInstalled ]; then 
      packagesMissing="TRUE"
    fi
    
    # -- (RH6 only) check 32 bit libpng.i686 package for DM (may not be needed for server)
    # could also get libpng-devel.i686 if required
    # THIS PACKAGE IS NOT NEEDED ON RH6
    ## =======================================================
    ##package=libpng.i686
    
    # -- (RH6 only) check 32 bit gtk2-engines.i686 package for DM (may not be needed for server)
    ## =======================================================
    package=gtk2-engines.i686
    checkRHPackageInstalled ${package}
    if [ "FALSE" == $rhPackageInstalled ]; then 
      packagesMissing="TRUE"
    fi
    
  fi
  
  
  if [ "TRUE" == $bRH7flag ]; then
    echo "+++ check Red Hat 7 specific packages"
    
    # -- (RH7 only) check 32 bit PackageKit-gtk3-module.i686 package for DM (may not be needed for server)
    ## =======================================================
    package=PackageKit-gtk3-module.i686
    checkRHPackageInstalled ${package}
    if [ "FALSE" == $rhPackageInstalled ]; then 
      packagesMissing="TRUE"
    fi
    
    # -- (RH7 only) check 32 bit libpng12.i686 package for DM (may not be needed for server)
    ## =======================================================
    package=libpng12.i686
    checkRHPackageInstalled ${package}
    if [ "FALSE" == $rhPackageInstalled ]; then 
      packagesMissing="TRUE"
    fi
  fi

fi

if [ "TRUE" == $packagesMissing ]; then
  echo "ERROR: script "${scriptName}" at least one required preReq RH package is missing"
  exit 1
else
  echo "all required preReq RH package present"
fi


##########################################################################
######### ensure custom LDAP properties have been set ####################
######### Only check on WAS nodes ########################################
echo ${localTags} | grep WAS
if [ $? -eq 0 ]; then
   if [ "$LDAPPROP" == "CustomLDAP" ]; then
      echo "check custom LDAP properties have been set"
      checkFileExists $AUTO_PREP/CustomLDAP/Custom_LDAPSecurity.properties
   fi
fi

################################################################################
######### ensure Database creation properties have been set ####################
echo "check Database creation properties have been set"
if [ "$DB_VENDOR" == "oracle" ];then
	checkFileExists $AUTO_PREP/DatabaseParameters/oracleParameter.properties
fi
if [ "$DB_VENDOR" == "db2" ]; then
	checkFileExists $AUTO_PREP/DatabaseParameters/DBParameter.properties
fi

####################################################################
######### ensure user credentials have been set ####################
echo "check user credentials have been set"
checkFileExists $AUTO_PREP/UserParameters/userCreds.properties
chmod 600 $AUTO_PREP/UserParameters/userCreds.properties
ls -la $AUTO_PREP/UserParameters

####################################################################
######### ensure DB2 user name meets the requirement ###############
. $AUTO_PREP/UserParameters/userCreds.properties
sh $AUTO_PREP/checkID.sh $ibmQseInst_User
if [ $? -eq 0 ]; then
   echo "Passed test for id [$ibmQseInst_User]"
else
   exit 1
fi

sh $AUTO_PREP/checkID.sh $ibmQseInst_Group
if [ $? -eq 0 ]; then
   echo "Passed test for group name [$ibmQseInst_Group]"
else
   exit 1
fi

####################################################################
######### ensure media properties have been set ####################
if [ -f $AUTO_PREP/MediaParameters/media.properties ]; then
  echo "media properties set"
else

  if [ "pipeline" == $mediaPropertiesType ]; then
    echo "using pipeline media properties"
    checkFileExists $AUTO_PREP/MediaParameters/media.pipeline
    cp $AUTO_PREP/MediaParameters/media.pipeline $AUTO_PREP/MediaParameters/media.properties
  elif [ "nfs" == $mediaPropertiesType ]; then
    echo "using nfs media properties"
    checkFileExists $AUTO_PREP/MediaParameters/media.nfs
    cp $AUTO_PREP/MediaParameters/media.nfs $AUTO_PREP/MediaParameters/media.properties
  elif [ "local" == $mediaPropertiesType ]; then
    echo "using local media properties"
    checkFileExists $AUTO_PREP/MediaParameters/media.local
    cp $AUTO_PREP/MediaParameters/media.local $AUTO_PREP/MediaParameters/media.properties
  else
    "ERROR: script "${scriptName}" do not support [${mediaPropertiesType}] media properties"
    exit 1
  fi
  checkFileExists $AUTO_PREP/MediaParameters/media.properties
  ######### add the version properties to the media properties####################
  checkFileExists $AUTO_PREP/MediaParameters/mediaVersion.properties
  cat $AUTO_PREP/MediaParameters/mediaVersion.properties>> $AUTO_PREP/MediaParameters/media.properties
fi
ls -la $AUTO_PREP/MediaParameters


#############################################
#########  source media properties  ######### 
. $AUTO_PREP/MediaParameters/media.properties


if [ "pipeline" == $mediaPropertiesType ]; then
  echo "get pipeline media credentials"

  ######### other credentials that change every 90 days ####################
  mkdir -p $AUTO_PREP/RATIONAL_SVT_IM_KEYRING
  cd $AUTO_PREP/RATIONAL_SVT_IM_KEYRING
  sh $AUTO_PREP/MediaParameters/doWget.sh ${mediaServer}/${imRepoCredRoot}/IM.Cred
  if [ -f IM.Cred ]
  then
      echo "file now exists, downloaded IM.Cred successful"
  else
      echo "ERROR: script "${scriptName}" IM.Cred does not exist, download failed"
      exit 1
  fi
  
  sh $AUTO_PREP/MediaParameters/doWget.sh ${mediaServer}/${imRepoCredRoot}/password.txt
  if [ -f password.txt ]
  then
      echo "file now exists, downloaded password.txt successful"
  else
      echo "ERROR: script "${scriptName}" password.txt does not exist, download failed"
      exit 1
  fi
  
  
  
  ####################################
  ## move credentials 
  echo "Move credentials files to [${finalCredentialsDir}]"
  
  # ensure the real credentials folder exists
  checkAndCreateFolder ${finalCredentialsDir}
  
  moveFile IM.Cred ${finalCredentialsDir}
  moveFile password.txt ${finalCredentialsDir}
  
  chmod 777 ${finalCredentialsDir}/*
  ls -la ${finalCredentialsDir}

fi

 

################################################
######### any other checks go in here ##########
################################################




#########  if required mount the network drive  ######### 
## has to be last as it cannot be done twice without manual clean up
if [ "nfs" == $mediaPropertiesType ]; then
  echo "mount the network drive [${mediaServer}:${mediaServerFolder}] [${mediaRoot}]"

  ## error if already mounted
  mount | grep "^${mediaServer}:${mediaServerFolder}.*${mediaRoot}"
  if [ $? -gt 0 ]; then
    echo "matching mount does not currently exist"
  else
    echo "ERROR: script "${scriptName}" matching mount already exists"
    echo "list of current mounts, one of which clashes!"
    mount
    exit 1
  fi
  
  mount | grep "on.*${mediaRoot}"
  if [ $? -gt 0 ]; then
    echo "mount of same name does not currently exist"
  else
    echo "ERROR: script "${scriptName}" mount [${mediaRoot}] already exists"
    echo "list of current mounts, one of which clashes!"
    mount
    exit 1
  fi
  
  
  # mount the network share
  mkdir -p ${mediaRoot}
  command="mount -t nfs ${mediaServer}:${mediaServerFolder} ${mediaRoot}"
  echo $!{command} ${command}
  ${command}
  
  # test the mount happened
  mount | grep "^${mediaServer}:${mediaServerFolder}.*${mediaRoot}"
  if [ $? -gt 0 ]; then
    echo "ERROR: script "${scriptName}" failed to mount drive"
    exit 1
  else
    echo "mount successfully created and contains"
    ls -la ${mediaRoot}
  fi
  result=$(getFolderPermission ${mediaRoot})
  if [ $? == 0 ]; then
        if [ "$result" -ge "744" ];then
                echo "${mediaRoot} has correct permission: [$result]"
        else
                echo "ERROR: Wrong permission set for directory [${mediaRoot}], current setting is [$result]"
                exit 1
        fi
  else
        echo "ERROR: Couldn't get the permission for [${mediaRoot}]"
        exit 1
  fi
fi


####################################
## EOF
echo "-- EOF [${scriptName}] SUCCESS --"
exit 0