#!/bin/sh
#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2015. All Rights Reserved.
#
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************

########################################################
#
# Script : doWget.sh
# Purpose :
# - Decrypt credentials {$user, $password}
# - Save ~/.wgetrc in ~/.wgetrc_Temp
# - Create new version of ~/.wgetrc
#       http-user=$user
#       http-password=$password
# - Run WGET
#       if(bCheckCertificate = true) wget nv  sourceFileUrl 
#       if(bCheckCertificate = false) wget -nv --no-check-certificate sourceFileUrl 
# - Put ~/.wgetrc_Temp back to ~/.wgetrc
#
#
########################################################

scriptName=$0

sourceUrl=$1
echo "sourceUrl [${sourceUrl}]"

#################
## constants

# get env properties
envProp=/etc/env.properties
. $envProp

# get credentials
userCreds=$AUTO_PREP/UserParameters/userCreds.properties
. $userCreds

# get media properties
mediaProp=$AUTO_PREP/MediaParameters/media.properties
. $mediaProp

wgetrcFile=~/.wgetrc
wgetrcOrigFile=${wgetrcFile}_orig

##############
## 
## FUNCTIONS
##
###############

## usage exitWithErrMsg "tempString"
exitWithErrMsg()
{
  tempString=$1
  echo "-- ERROR [${scriptName}] --"
  echo "-- message: $tempString"
  
  cleanUpTempFiles
  
  exit 1
}


## usage cleanUpTempFiles
cleanUpTempFiles()
{

  if [ "https" == ${wgetProtocol} ]; then
    
    ################
    ## delete wget control file
    deleteFile ${wgetrcFile}
    
    ################
    ## return wget control file
    if [ -f ${wgetrcOrigFile} ]; then
      moveFile ${wgetrcOrigFile} ${wgetrcFile}
    fi
  
  fi

}


## usage checkBoolean bFlag
## -- accepts true, false, TRUE, FALSE (and a mismatch of case!)
checkBoolean()
{

  tempFlag=$1
  # convert to lowercase
  convertedFlag=`echo $tempFlag | tr [A-Z] [a-z]`
  
  if [ "true" == ${convertedFlag} ] || [ "false" == ${convertedFlag} ];then
    echo "[${tempFlag}] is boolean"
  else
    errMsg="[${tempFlag}] is not boolean"
    exitWithErrMsg "${errMsg}"
  fi

}


## usage checkProtocol sgetProtocol
## -- accepts http, https
checkProtocol()
{

  tempProtocol=$1
  if [ "http" == ${tempProtocol} ] || [ "https" == ${tempProtocol} ];then
    echo "[${tempProtocol}] is valid"
  else
    errMsg="[${tempProtocol}] is not a valid protocol"
    exitWithErrMsg "${errMsg}"
  fi

}


## usage deleteFile fileSource
deleteFile()
{

  tempFile=$1

  echo "delete [${tempFile}]"
  checkFileExists ${tempFile}
  
  rm -f ${tempFile}
  echo "deleting..."
  if [ -f ${tempFile} ]; then
    errMsg="File still exists"
    exitWithErrMsg "${errMsg}"
  else
    echo "File deleted"
  fi

}


## usage moveFile fileSource fileTarget
moveFile()
{

  tempSourceFile=$1
  tempTargetFile=$2

  echo "move [${tempSourceFile}] to [${tempTargetFile}]"
  checkFileExists ${tempSourceFile}
  
  mv ${tempSourceFile} ${tempTargetFile}
  echo "moving..."
  checkFileExists ${tempTargetFile}
  
}


## usage checkFileExists tempFile
checkFileExists()
{
  tempFile=$1
  if [ -f $tempFile ]; then
    echo "file [$tempFile] exists"
  else
    errMsg="[$tempFile] does not exist"
    exitWithErrMsg "${errMsg}"
  fi
}


##############
## 
## MAIN
##
###############


##################
## check some properties are valid


checkBoolean $checkWgetCertificate
# convert to lowercase for later use
checkWgetCertificate="$(echo $checkWgetCertificate | tr '[A-Z]' '[a-z]')"

checkProtocol ${wgetProtocol}


if [ "https" == ${wgetProtocol} ]; then
  # need to use credentials

  tempUser=$mediaUser
  tempPassword=`echo ${mediaPassword} | base64 -d`
  echo "user [${tempUser}] password [**XX**]"
  
  ################
  ## Save wget control file
  if [ -f wgetrcFile  ]; then
    moveFile ${wgetrcFile} ${wgetrcOrigFile}
  fi
  
  ################
  ## Write temporary /etc/wgetrc
  touch ${wgetrcFile}
  echo http-user=${tempUser}>> ${wgetrcFile}
  echo http-password=${tempPassword}>> ${wgetrcFile}
  chmod 644 ${wgetrcFile}

fi


#################
## run WGET
echo "download [${sourceUrl}]"
if [ "true" == ${checkWgetCertificate} ];then
  command="wget -nv "${sourceUrl}
else
  command="wget -nv --no-check-certificate "${sourceUrl}
fi
echo "executing command [${command}]"
${command}
rc=$?
if [ ${rc} -gt 0 ]; then
  errMsg="errCode [${rc}] Failed to download [${sourceUrl}]"
  exitWithErrMsg "${errMsg}"
else
  echo "download successful"
fi


#################
# ensure no credentials left lying around
cleanUpTempFiles


################
## === finished ====
echo "-- EOF [${scriptName}] SUCCESS --"
exit 0


