#!/bin/bash
#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2015. All Rights Reserved.
#
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************
######################################################
# parse the new global parameter
# and update the local property file
#
#
# input format is elemnent1;element2;element3 etc
# where element is hostname:appList
#
# will update the following local property file entries
#
# DB_HOSTNAME
# DB_VENDOR
#
# hosts
# ihshost
# dwahost
# NUMBER_OF_CUSTOM_NODES
# NUMBER_OF_DWA_NODES
# NODE{#}_HostName
# NODE{#}_AppList
# NODE{#}_NodeName
#
# JTSURLHost
# tableList
#
# and check these rules
# ---------------------------------------------------
# 1. only one IHS must exists (not optional yet!)
# 2. only one database must exist
# 3. JTS must exist
# 4. only one instance of any application can exist
# 5. if CMenable is TRUE
#      ensure LDX and GC are present
#      ensure activation key is set

######################################################


## FUNCTION


# update property file with the new key:value pair
updateServerPropertiesFile()
{
  origFile=$1
  tempKey=$2
  newValue=$3
  
  tempFile=${origFile}.tmp
  matchKey=${tempKey}"="
  
  # does key already exist?
  cat ${origFile} | grep ${matchKey} > /dev/NUL
  if [ $? -gt 0 ]; then
    # key does not exist, so add it to the file
    echo "add new key ["${tempKey}"] ["${newValue}"]"
    echo ${tempKey}"="${newValue}>> ${origFile}
  else
    # key does exist, so update it
    echo "update existing key with ["${tempKey}"] ["${newValue}"]"
    # lines starting with # should not be updated
    sed "s/^ *${tempKey}=.*/${tempKey}=$newValue/g" ${origFile} > ${tempFile}
    mv ${tempFile} ${origFile}
  fi
}


## usage checkBoolean bFlag
## -- accepts true, false, TRUE, FALSE (and a mismatch of case!)
checkBoolean()
{

  tempFlag=$1
  # convert to lowercase
  convertedFlag=`echo $tempFlag | tr [A-Z] [a-z]`
  
  if [ "true" == ${convertedFlag} ] || [ "false" == ${convertedFlag} ];then
    echo "[${tempFlag}] is boolean"
  else
    echo "[${tempFlag}] is not boolean"
    exit 1
  fi

}


## usage errorWrongNumberOfInputs
errorWrongNumberOfInputs()
{
  echo "ERROR: wrong number of input parameters, expected [$expectedInputParameterCount] not [$numberOfInputs]"
  echo "from command [$0] [$allInputs]" 
  echo " "
  echo "Usage: [globalTagListing] [(bool) CM enable] [CM activation key]"
  exit 1
}


##########################################
## MAIN
##########################################

scriptName=$0
numberOfInputs=$#
allInputs=$*
expectedInputParameterCount=5

globalTagListing=$1
CMenable=$2
LDAPPROP=$3
DB_VENDOR=$4
CMActivationKey=$5


# check inputs
if [ $((expectedInputParameterCount-1 )) -eq $numberOfInputs ]; then
  # set activation key to be blank
  CMActivationKey=""
  echo "globalTagListing ["$globalTagListing"]"
  echo "CMenable ["$CMenable"]"
  echo "CMActivationKey ["$CMActivationKey"]"
elif [ $expectedInputParameterCount -ne $numberOfInputs ]; then
  errorWrongNumberOfInputs
else
  echo "globalTagListing ["$globalTagListing"]"
  echo "CMenable ["$CMenable"]"
  echo "CMActivationKey ["$CMActivationKey"]"
  echo "LDAPPROP ["$LDAPPROP"]"
  echo "DB_VENDOR ["$DB_VENDOR"]"
fi

checkBoolean $CMenable
# convert to uppercase for later use
CMenable="$(echo $CMenable | tr '[a-z]' '[A-Z]')"


propFile="/etc/env.properties"
echo "check "${propFile}" exists"
if [ -f "${propFile}" ]; then
  echo "file exists"
else
  echo "ERROR: file does not exist"
  exit 1
fi

propFilePermissions=`stat -c %a ${propFile}`
echo "capture file ["${propFile}"] permissions ["${propFilePermissions}"]"

ProcessTagging_DIR=`pwd -P`
echo "" >> $propFile
echo ProcessTagging_DIR=${ProcessTagging_DIR}>> $propFile
echo ProcessTagging_DIR: [${ProcessTagging_DIR}]

hostList=""
tagList=""
appList=""
allAppsList=""
individualAppsList=""
duplicateAppFlag="FALSE"
tableList=""
# rs does not have a table
validTables="vvc,dm,lqe,dcc,relm,ldx,gc,qm,rm,ccm,jts"

# start counting at 1 rather than 0
wasNodeCount=1
dwaNodeCount=1

ihsPresent="FALSE"
ihsCount=0
dbPresent="FALSE"
dbCount=0
jtsPresent="FALSE"
ldxPresent="FALSE"
gcPresent="FALSE"

localHost=`hostname -f`
echo localHost [${localHost}]

tempList=${globalTagListing//;/ }
# split list into individual elements
for element in $tempList
do
  elementList=${element//:/ }
  # use array to access hostname and arg list from elementList
  elementArray=($elementList)

  # ihs treated differently
  if [[ ${elementArray[1]} == *IHS* ]]; then
    # ihs server assume no WAS apps present
    ihsPresent="TRUE"
    ihsCount=$(( ihsCount+1 ))
    ihshost=${elementArray[0]}
    IHS_AppList=${elementArray[1]}
    updateServerPropertiesFile $propFile "ihshost" ${ihshost}
    # do not update the nodeCounts or the hostList
  
  # has to be before *DWA* to avoid an unwanted match
  elif  [[ ${elementArray[1]} == *DWAINTEROP* ]]; then
    # dwa interop server assume no WAS apps present
    DWAINTEROP_HostName=${elementArray[0]}
    DWAINTEROP_AppList=${elementArray[1]}
    # do not update the nodeCounts or the hostList
  
  elif  [[ ${elementArray[1]} == *DWA* ]]; then
    # dwa server assume no WAS apps present
    dwahost=${elementArray[0]}
    DWA_AppList=${elementArray[1]}
    updateServerPropertiesFile $propFile "dwahost" ${dwahost}
    # do not update the wasNodeCounts, but do update the hostList as it is needed for certificates
    hostList=${hostList},${elementArray[0]}
    dwaNodeCount=$(( dwaNodeCount+1 ))
  
  elif  [[ ${elementArray[1]} == *DOORSD* ]]; then
    # doorsd (database) server assume no WAS apps present
    DOORSD_HostName=${elementArray[0]}"]"
    DOORSD_AppList=${elementArray[1]}
    # do not update the nodeCounts or the hostList
  
  elif  [[ ${elementArray[1]} == *DB* ]]; then
    # database server (oracle or db2) assume no WAS apps present
    dbPresent="TRUE"
    dbCount=$(( dbCount+1 ))
    DB_HOSTNAME=${elementArray[0]}
    # needs to be lowercase
    DB_VENDOR=`echo ${DB_VENDOR} | tr [A-Z] [a-z]`
    updateServerPropertiesFile $propFile "DB_HOSTNAME" ${DB_HOSTNAME}
    updateServerPropertiesFile $propFile "DB_VENDOR" ${DB_VENDOR}
    # do not update the nodeCounts or the hostList
    
  else
    appList=""
    # was server assume WAS apps present
    #NODE${wasNodeCount}_HostName=${elementArray[0]}
    #NODE${wasNodeCount}_TagList=${elementArray[1]}
    #NODE${wasNodeCount}_NodeName=${elementArray[2]} if it exists 
    updateServerPropertiesFile $propFile "NODE${wasNodeCount}_HostName" ${elementArray[0]}
    if [[ 3 -eq ${#elementArray[@]} ]]; then 
      # also have the WAS NODENAME
      updateServerPropertiesFile $propFile "NODE${wasNodeCount}_NodeName" ${elementArray[2]}
    fi
    
    # need to extract the APP list from the TAG list
    # the JTS server hostname
    # and create a list of database tables to create
    
    tagList=${elementArray[1]//,/ }
    # split list into individual elements
    for tag in $tagList
    do
      # convert tag to lowercase
      tag="$(echo $tag | tr '[A-Z]' '[a-z]')"
      # create this server appList
      if [[ $tag != was ]]; then
        if [[ $tag == jrs ]]; then
          tag="rs"
        fi

        appList=${appList},${tag}
        
        # also update the individualAppsList
        appExistsFlag="FALSE"
        currentAppList=${individualAppsList//,/ }
        for cApp in $currentAppList
        do
          if [[ $tag == $cApp ]]; then
            appExistsFlag="TRUE"
            duplicateAppFlag="TRUE"
          fi
        done
        
        if [ "FALSE" == $appExistsFlag ]; then
          # not in current list, so add it
          individualAppsList=${individualAppsList},${tag}
        fi
      
        # update the tableList if $tag is not in the list already and a valid table
        tableExistsFlag="FALSE"
        currentTableList=${tableList//,/ }
        for cTable in $currentTableList
        do
          if [[ $tag == $cTable ]]; then
            tableExistsFlag="TRUE"
          fi
        done
        
        if [ "FALSE" == $tableExistsFlag ]; then
          # not in current list, now check it is a valid table
          validTablesList=${validTables//,/ }
          for vTable in $validTablesList
          do
            if [[ $tag == $vTable ]]; then
              # add to the tableList
              tableList=${tableList},${tag}
            fi
          done
        fi
      fi
      # is it jtshost?
      if [[ $tag == jts ]]; then
        jtsPresent="TRUE"
        JTSURLHost=${elementArray[0]}
        updateServerPropertiesFile $propFile "JTSURLHost" ${JTSURLHost}
      fi
      # is it ldx?
      if [[ $tag == ldx ]]; then
        ldxPresent="TRUE"
      fi
      # is it gc?
      if [[ $tag == gc ]]; then
        gcPresent="TRUE"
      fi
    done
    # remove trailing comma and convert to lowercase
    appList=`echo ${appList} | cut -c 2-`
    updateServerPropertiesFile $propFile "NODE${wasNodeCount}_AppList" ${appList}
    # update the list of all apps present
    allAppsList=${allAppsList}","${appList}
    
    hostList=${hostList},${elementArray[0]}
    wasNodeCount=$(( wasNodeCount+1 ))

  fi
done
# -- hostnames --
# remove leading "," character
hosts=`echo ${hostList} | cut -c 2-`
updateServerPropertiesFile $propFile "hosts" ${hosts}

# -- database tables --
# remove trailing comma and convert to lowercase
tableList=`echo ${tableList} | cut -c 2-`
updateServerPropertiesFile $propFile "databaseList" ${tableList}

# -- apps --
# remove leading comma
allAppsList=`echo ${allAppsList} | cut -c 2-`
individualAppsList=`echo ${individualAppsList} | cut -c 2-`
# save every app instance
updateServerPropertiesFile $propFile "appList" ${individualAppsList}
updateServerPropertiesFile $propFile "allAppsList" ${allAppsList}

# -- custom / dwa nodes --
# as count started at 1 the real number is one less
NUMBER_OF_CUSTOM_NODES=$(( wasNodeCount-1 ))
updateServerPropertiesFile $propFile "NUMBER_OF_CUSTOM_NODES" ${NUMBER_OF_CUSTOM_NODES}

NUMBER_OF_DWA_NODES=$(( dwaNodeCount-1 ))
updateServerPropertiesFile $propFile "NUMBER_OF_DWA_NODES" ${NUMBER_OF_DWA_NODES}

# -- CMenable --
updateServerPropertiesFile $propFile "EnableCM" ${CMenable}
updateServerPropertiesFile $propFile "CMActivationKey" ${CMActivationKey}
updateServerPropertiesFile $propFile "LDAPPROP" ${LDAPPROP}

# -- no more property file update left, so put original permissions back --
echo "ensure file ["${propFile}"] permissions ["${propFilePermissions}"]"
chmod ${propFilePermissions} ${propFile}
ls -la ${propFile}

# -- check rules --
# 1. only one IHS must exists (not optional yet!)
# 2. only one database must exist
# 3. JTS must exist
# 4. only one instance of any application can exist
# 5. if CMenable is TRUE
#      ensure LDX and GC are present
#      ensure activation key is set

echo "--- check system rules ---"
rulesInValidated="FALSE"


# 1. only one IHS must exists (not optional yet!)
echo "check only 1 IHS allowed rule"
if [ "TRUE" == $ihsPresent ]; then
  # ihs exists
  if [ 1 == $ihsCount ]; then
    echo "--> rule validated"
  else
    echo "-- ERROR: script "${scriptName}" system has too many ["${ihsCount}"] IHS servers --> rule invalidated"
    rulesInValidated="TRUE"
  fi
else
  echo "-- ERROR: script "${scriptName}" system has no ihs present --> rule invalidated"
  rulesInValidated="TRUE"
fi


# 2. only one database must exist
echo "check only 1 DATABASE allowed rule"
if [ "TRUE" == $dbPresent ]; then
  # db exists
  if [ 1 == $dbCount ]; then
    echo "--> rule validated"
  else
    echo "-- ERROR: script "${scriptName}" system has too many ["${dbCount}"] DATABASE servers --> rule invalidated"
    rulesInValidated="TRUE"
  fi
else
  echo "-- ERROR: script "${scriptName}" system has no database present --> rule invalidated"
  rulesInValidated="TRUE"
fi

# 3. JTS must exist
echo "check JTS present rule"
if [ "TRUE" == $jtsPresent ]; then
  # jts exists
  echo "--> rule validated"
else
  echo "-- ERROR: script "${scriptName}" system has no JTS application present --> rule invalidated"
  rulesInValidated="TRUE"
fi

# 4. only one instance of any application can exist
echo "check no duplicate APPS rule"
if [ "FALSE" == $duplicateAppFlag ]; then
  echo "--> rule validated"
else
  echo "-- ERROR: script "${scriptName}" duplicate APPS exist --> rule invalidated"
  # count instances of each app
  splitIndividualAppsList=${individualAppsList//,/ }
  splitAllAppsList=${allAppsList//,/ }
  for indApp in $splitIndividualAppsList
  do
    appCount=0
    for allApp in $splitAllAppsList
    do
      if [ $indApp == $allApp ]; then
        appCount=$(( appCount+1 ))
      fi
    done
    echo "["$indApp"] count ["$appCount"]"
  done
  rulesInValidated="TRUE"
fi


# 5. if CMenable is TRUE
#      ensure LDX and GC are present
#      ensure activation key is set
if [ "TRUE" == $CMenable ]; then
  echo "CM enabled, check LDX and GC applications are present rule"
  if [ "TRUE" == $ldxPresent ] && [ "TRUE" == $gcPresent ]; then
    echo "--> rule validated"
  else
    echo "-- ERROR: script "${scriptName}" LDX present ["${ldxPresent}"] or GC present ["${gcPresent}"] --> rule invalidated"
    rulesInValidated="TRUE"
  fi
  echo "CM enabled, check CMActivationKey rule"
  if [ "XX" == "X"${CMActivationKey}"X" ]; then
    echo "-- ERROR: script "${scriptName}" CMActivationKey missing ["${CMActivationKey}"]  --> rule invalidated"
    rulesInValidated="TRUE"
  else
    echo "--> rule validated"
  fi 
  
fi

# confirm all is valid or exit
if [ "FALSE" == $rulesInValidated ]; then
  echo "system rules ALL VALID"
else
  echo "-- ERROR: script "${scriptName}" ONE OR MORE RULES INVALID"
  exit 1
fi


echo "-- EOF [${scriptName}] --"
exit 0
