#!/bin/bash

#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2016. All Rights Reserved.
#
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************

#################################################################################
##
## Name: InstallMaintenanceFiles.sh
##
## Execute the script
## ------------------
## sh .\InstallMaintenanceFiles.sh
#


## aim
## ---

## this script does the following 
## ------------------------------
## 1. checks rules to confirm it is valid to install the iFix
## 2. create / assemble the iFix media (zip) ready for the "engine to use"
## 3. call child script to install the iFix
##
## assumptions
## -----------
## 1. there is already an install done [eg 6.0 GA]
## 2. nothing has been deployed into WAS
## 3. no inputs, control comes from /etc/env.properties and $APPS (tagging system)
## 4. iFix rules have already been checked
##
#################################################################################


##############
## FUNCTIONS #
##############


## usage exitWithOkMsg tempString
exitWithOkMsg()
{
  tempString=$1
  echo "-- message: ${tempString}"
  echo "+++ SUCCESS : EOF [${scriptName}] +++"
  exit 0
}


## usage exitWithErrMsg tempString
exitWithErrMsg()
{
  tempString=$1
  echo "-- ERROR [${scriptName}] --"
  echo "-- message: ${tempString}"
  exit 1
}


## usage exitWithWarnMsg tempString
exitWithWarnMsg()
{
  tempString=$1
  echo "-- WARNING [${scriptName}] --"
  echo "-- message: ${tempString}"
  exit 0
}


## usage checkFileExists tempFile
checkFileExists()
{
  tempFile=$1
  if [ -f ${tempFile} ]; then
    echo "file [${tempFile}] exists"
  else
    errMsg="[${tempFile}] does not exist"
    exitWithErrMsg "${errMsg}"
  fi
}


## usage checkFolderExists tempFolder
checkFolderExists()
{
  tempFolder=$1
  if [ -d ${tempFolder} ]; then
    echo "folder [${tempFolder}] exists"
  else
    errMsg="[${tempFolder}] does not exist"
    exitWithErrMsg "${errMsg}"
  fi
}


## make a new folder including any missing folders in the path
makeFolderFullPath()
{
  newFolder=$1
  echo "make new folder (inc path) [${newFolder}]"
  mkdir -p ${newFolder}
  checkFolderExists ${newFolder}
}


## usage download file from "jazzweb" to current folder
downloadHttpZip()
{
  fileUrl=$1
  filename=$(basename $fileUrl)

  sh $AUTO_PREP/MediaParameters/doWget.sh $fileUrl
  if [ $? == 0 ]
  then
    echo "download complete"
    checkFileExists $filename
  else
    errMsg="failed to download file ["${filename}"]"
    exitWithErrMsg "$errMsg"
  fi
}


# copy zipfile from "nfs" to current folder
copyZip()
{
  file=$1
  filename=$(basename $file)
  echo "copy ["${file}"] to ["${filename}"]"
  cp $file $filename
  if [ $? == 0 ]
  then
    echo "copy complete" 
    checkFileExists $filename
  else
    errMsg="failed to copy file ["${filename}"]"
    exitWithErrMsg "$errMsg"
  fi
}


## usage copyFile (force) from current folder to folder
copyFile()
{
  tempFile=$1
  tempFolder=$2
  echo "copy ["${tempFile}"] to ["${tempFolder}/${tempFile}"]"
  cp -f ${tempFile} ${tempFolder}/${tempFile}
  if [ $? == 0 ]; then
    echo "copy complete" 
    if [ -f ${tempFolder}/${tempFile} ]; then
      echo "copy successful, file exists"
    else
      errMsg="file does not exist"
      exitWithErrMsg "${errMsg}"
    fi
  else
    errMsg="copy failed"
    exitWithErrMsg "${errMsg}"
  fi
}


## usage moveFile file to new location
moveFile()
{
  oldFileName=$1
  newFileName=$2
  echo "move ["${oldFileName}"] to ["${newFileName}"]"
  mv -f ${oldFileName} ${newFileName}
  if [ $? == 0 ]; then
    echo "move complete" 
    if [ -f ${newFileName} ]; then
      echo "move successful, file exists"
    else
      errMsg="file does not exist"
      exitWithErrMsg "${errMsg}"
    fi
  else
    errMsg="move failed"
    exitWithErrMsg "${errMsg}"
  fi
}


## usage deleteFile (force) from folder
deleteFile()
{
  tempFolder=$1
  tempFile=$2
  fileToDelete=${tempFolder}"/"${tempFile}
  
  if [ -f ${fileToDelete} ]; then
    echo "delete file ["${fileToDelete}"]"
    rm -f ${fileToDelete}
    if [ -f ${fileToDelete} ]; then
      errMsg="failed to delete file"
      exitWithErrMsg "${errMsg}"
    else
      echo "succesfully deleted [${fileToDelete}]"
    fi
  else
    echo "file [${fileToDelete}] does not exist, so cannot be deleted"
  fi
}


## usage recursively deleteFolder (force)
deleteFolder()
{
  tempFolder=$1
  
  if [ -d ${tempFolder} ]; then
    echo "delete folder ["${tempFolder}"]"
    rm -Rf ${tempFolder}
    if [ -d ${tempFolder} ]; then
      errMsg="failed to delete folder"
      exitWithErrMsg "${errMsg}"
    else
      echo "succesfully deleted [${tempFolder}]"
    fi
  else
    echo "folder [${tempFolder}] does not exist, so cannot be deleted"
  fi
}


## usage zip contents of current folder to target zipfile
zipFolder()
{
  zipFile=$1
  targetDir=$2
  
  targetZipFile=${targetDir}"/"${zipFile}
  echo "recusively zipping current folder contents into [$targetZipFile]"
  zip -r $targetZipFile *
  if [ $? == 0 ]; then
    echo "zip successful" 
  else
    errMsg="failed to create zip  ["${targetZipFile}"]"
    exitWithErrMsg "$errMsg"
  fi
}


listZipContents()
{
  zipFile=$1
  
  unzip -l ${zipFile}
  if [ $? == 0 ]; then
    echo "  zip listed successfully" 
  else
    errMsg="failed to list contents of ["${zipFile}"]"
    exitWithErrMsg "$errMsg"
  fi
}


## usage unzip contents to target folder
unzipZip()
{
  zipFile=$1
  targetDir=$2
  
  unzip ${zipFile} -d ${targetDir}
  if [ $? == 0 ]; then
    echo "unzip successful" 
  else
    errMsg="failed to unzip  ["${zipFile}"]"
    exitWithErrMsg "$errMsg"
  fi
}


## usage unzip partial contents to target folder
unzipZipPartial()
{
  zipFile=$1
  targetDir=$2
  sectionReference=$3
  
  unzip -j ${zipFile} ${sectionReference} -d ${targetDir}
  if [ $? == 0 ]; then
    echo "unzip successful" 
  else
    errMsg="failed to unzip section ["${sectionReference}"] from ["${zipFile}"]"
    exitWithErrMsg "$errMsg"
  fi
}


## calculate the application release eg 6.0.0
setAppRelease()
{
  # extract the prefix and then the release
  appMediaContent=`echo ${appMediaRoot} | cut -c -2`
  if [ "SS" == $appMediaContent ]; then
    # SSE
    appRelease=`echo ${appMediaRoot} | cut -c 4-`
  elif  [ "CL" == $appMediaContent ]; then
    # CLM
    appRelease=`echo ${appMediaRoot} | cut -c 4-`
  elif  [ "CE" == $appMediaContent ]; then
    appRelease=`echo ${appMediaRoot} | cut -c 3-`
  else
    errMsg="unsupported prefix ["${appMediaContent}"] in ["${appMediaRoot}"]"
    exitWithErrMsg "$errMsg"
  fi
  echo "appMediaRoot [${appMediaRoot}], appMediaContent [${appMediaContent}], appRelease [${appRelease}]"
}


## calculate the maintenance stream id eg 6.0.0.0
# -- covers CLM not sure about DM
CLMsetMaintStreamID()
{
  # translate appRelease into the maintenance stream string
  tempAppRelease=$1
  
  case "$tempAppRelease" in
    603) maintClmStreamID="6.0.3.0"
       ;;
    602) maintClmStreamID="6.0.2.0"
       ;;
    601) maintClmStreamID="6.0.1.0"
       ;;
    600) maintClmStreamID="6.0.0.0"
       ;;
    60) maintClmStreamID="6.0.0.0"
       ;;
    *) errMsg="no maintenance stream for [${tempAppRelease}]"
       exitWithErrMsg "$errMsg"
       ;;
  esac
  
  echo "tempAppRelease [${tempAppRelease}], maintClmStreamID [${maintClmStreamID}]"
}


# update property file with the new key:value pair
updateServerPropertiesFile()
{
  origFile=$1
  tempKey=$2
  newValue=$3
  
  tempFile=${origFile}.tmp
  matchKey=${tempKey}"="
  
  # does key already exist?
  cat ${origFile} | grep ${matchKey} > /dev/NUL
  if [ $? -gt 0 ]; then
    # key does not exist, so add it to the file
    echo "add new key ["${tempKey}"] ["${newValue}"]"
    echo ${tempKey}"="${newValue}>> ${origFile}
  else
    # key does exist, so update it
    echo "update existing key with ["${tempKey}"] ["${newValue}"]"
    # must match from the start of the line, using "|" instead of "/" in case $value is a path!
    sed "s|^ *${tempKey}=.*|${tempKey}=$newValue|g" ${origFile} > ${tempFile}
    mv ${tempFile} ${origFile}
  fi
}





############
### MAIN ###
############

## assume we run from the script's directory containing
##  InstallMaintenenceFiles.sh
## use the following from env.properties
#   $CLM_BUILDROOTURL
#   $CLM_BUILD_ID
#   $AUTO_PREP

scriptName=$0
workingDir=`pwd -P`

# get env properties
envProp=/etc/env.properties
. $envProp

# get media properties (eg $clmPatchZipSuffix)
mediaProp=$AUTO_PREP/MediaParameters/media.properties
. $mediaProp

# target folders for ZIP and WAR files
CLM_HOME=/opt/IBM/JazzTeamServer

# put all under a fixed location
localIFixHeadFolder=${QD_InternalMedia}"/iFixPatches"
#####################################################

# - will contain the iFix zip (either self assembled or supplied from a media server)
localIFixZipFolder=${localIFixHeadFolder}"/Zip"
# - contains the extracted content of the iFix zip as used by "the engine"
localIFixZipFolderFiles=${localIFixHeadFolder}"/Files"
# add to env.properties so the "engine" can use the value
updateServerPropertiesFile $envProp "localIFixZipFolderFiles" $localIFixZipFolderFiles


# - contains the assembled downloaded files ready for zipping
CLMlocalIFixDownloadedFilesFolder=${localIFixHeadFolder}"/CLM/Downloaded/Files"
# - contains intermediate additional files (zip)
CLMlocalIFixDownloadedAdditionalFilesFolder=${localIFixHeadFolder}"/CLM/Downloaded/Additional"
# - contains intermediate additional files (zip processing)
CLMlocalIFixDownloadedTempFolder=${localIFixHeadFolder}"/CLM/Downloaded/Temp"




## ======================================================================
## input parameters
# none


## ======================================================================
## set ${appRelease} for later use
setAppRelease


## =======================================================
## set control flags for later use

###############
## none
###############


## =======================================================
## RULES ##
# 1. iFix {iFixEnabled} must be enabled
# 2. iFix install selected eg not NONE {CLMiFixVersion} {DMiFixVersion} {DMMSiFixVersion}
# 3. server in scope must have an application in [${clmPatchableApps}]
## ========

echo "--- start checking it's valid to install the iFix rules ---"

# 1. iFix must be enabled
if [ "FALSE" == ${iFixEnabled} ]; then
    ## do nothing
    errMsg="not installing iFix as $!iFixEnabled [${iFixEnabled}]"
    exitWithWarnMsg "$errMsg"
else
  echo "potentially installing iFix as $!iFixEnabled [${iFixEnabled}]"
fi

# 2. iFix install(s) selected
if [ "NONE" == ${CLMiFixVersion} ]; then
  ## do nothing
  echo "not installing any CLM patches as $!CLMiFixVersion [${CLMiFixVersion}]"
  possibleCLMpatch=FALSE
else
  echo "potentially installing CLM patches as $!iFixVersion [${CLMiFixVersion}]"
  possibleCLMpatch=TRUE
fi
if [ "NONE" == ${DMiFixVersion} ]; then
  ## do nothing
  echo "not installing any DM patches as $!DMiFixVersion [${DMiFixVersion}]"
  possibleDMpatch=FALSE
else
  echo "potentially installing DM patches as $!DMiFixVersion [${DMiFixVersion}]"
  possibleDMpatch=TRUE
fi
if [ "NONE" == ${DMMSiFixVersion} ]; then
  ## do nothing
  echo "not installing any Rhapsody Model Server patches as $!DMMSiFixVersion [${DMMSiFixVersion}]"
  possibleDMMSpatch=FALSE
else
  echo "potentially installing Rhapsody Model Server patches as $!DMMSiFixVersion [${DMMSiFixVersion}]"
  possibleDMMSpatch=TRUE
fi

if [ "FALSE" == ${possibleCLMpatch} ] && [ "FALSE" == ${possibleDMpatch} ] && [ "FALSE" == ${possibleDMMSpatch} ]; then
    ## do nothing
    errMsg="no iFix patches to install"
    exitWithWarnMsg "$errMsg"
fi

# 3. server(s) in scope
if [ "TRUE" == ${possibleCLMpatch} ]; then
  clmPatchableApps="gc,dcc,relm,qm,rm,ccm,jts,lqe,ldx,rs"
  apps=${APPS//,/ }
  echo "apps [$apps]"
  patchableCLMServer=FALSE
  for a in $apps
  do
    vpList=${clmPatchableApps//,/ }
    for vp in $vpList
    do
      if [[ $a == $vp ]]; then
        patchableCLMServer=TRUE
      fi
    done
  done
  if [ "FALSE" == $patchableCLMServer ]; then
    echo "not installing CLM patches as no tag in [${APPS}] matches one in the CLM list [${clmPatchableApps}]"
    patchableCLMServer=FALSE
  else
    echo "potentially installing CLM patches as at least one tag in [${APPS}] matches one in the CLM list [${clmPatchableApps}]"
  fi
else
  patchableCLMServer=FALSE
fi

if [ "TRUE" == ${possibleDMpatch} ]; then
  dmPatchableApps="dm"
  apps=${APPS//,/ }
  echo "apps [$apps]"
  patchableDMServer=FALSE
  for a in $apps
  do
    vpList=${dmPatchableApps//,/ }
    for vp in $vpList
    do
      if [[ $a == $vp ]]; then
        patchableDMServer=TRUE
        if [ "TRUE" == ${possibleCLMpatch} ]; then
          patchableCLMServer=TRUE
        fi
      fi
    done
  done
  if [ "FALSE" == $patchableDMServer ]; then
    echo "not installing DM patches as no tag in [${APPS}] matches one in the DM list [${dmPatchableApps}]"
    patchableDMServer=FALSE
  else
    echo "potentially installing DM patches as at least one tag in [${APPS}] matches one in the DM list [${dmPatchableApps}]"
    if [ "TRUE" == ${patchableCLMServer} ]; then
      echo "also potentially installing additional CLM patches"
    fi
  fi
else
  patchableDMServer=FALSE
fi

if [ "TRUE" == ${possibleDMMSpatch} ]; then
  dmmsPatchableApps="dm"
  apps=${APPS//,/ }
  echo "apps [$apps]"
  patchableDMMSServer=FALSE
  for a in $apps
  do
    vpList=${dmmsPatchableApps//,/ }
    for vp in $vpList
    do
      if [[ $a == $vp ]]; then
        patchableDMMSServer=TRUE
        if [ "TRUE" == ${possibleCLMpatch} ]; then
          patchableCLMServer=TRUE
        fi
      fi
    done
  done
  if [ "FALSE" == $patchableDMMSServer ]; then
    echo "not installing Rhapsody Model Server patches as no tag in [${APPS}] matches one in the DMMS list [${dmmsPatchableApps}]"
    patchableDMMSServer=FALSE
  else
    echo "potentially installing Rhapsody Model Server patches as at least one tag in [${APPS}] matches one in the DMMS list [${dmmsPatchableApps}]"
    if [ "TRUE" == ${patchableCLMServer} ]; then
      echo "also potentially installing additional CLM patches"
    fi
  fi
else
  patchableDMMSServer=FALSE
fi

if [ "FALSE" == ${patchableCLMServer} ] && [ "FALSE" == ${patchableDMServer} ] && [ "FALSE" == ${patchableDMMSServer} ]; then
  ## do nothing
  errMsg="no tags match any of the iFix patch lists"
  exitWithWarnMsg "$errMsg"
else
  echo "patchableCLMServer [$patchableCLMServer] patchableDMServer [$patchableDMServer] patchableDMMSServer [$patchableDMMSServer]"
fi

echo "--- end of it's valid to install the iFix rules ---"


## =======================================================
## CHECK media setup##
## =================

#patchableCLMServer
#patchableDMServer
#patchableDMMSServer




if [ "CLM" == $iFixMediaType ]; then
  echo "using [${iFixMediaType}] media location type"
  if [ "TRUE" == ${patchableCLMServer} ]; then
    ## check build exists in env.properties
    if [ "X"$CLM_BUILD_ID"X" == "XX" ] || [ "X"$CLM_BUILDROOTURL"X" == "XX" ]; then
      errMsg="$!CLM_BUILDROOTURL [$CLM_BUILDROOTURL] or $!CLM_BUILD_ID [$CLM_BUILD_ID] not set!"
      exitWithErrMsg "$errMsg"
    else
      echo "using CLM repository, $!CLM_BUILDROOTURL [$CLM_BUILDROOTURL], $!CLM_BUILD_ID [$CLM_BUILD_ID]"
    fi
  fi
  
  if [ "TRUE" == ${patchableDMServer} ]; then
    echo "installing DM patch not supported"
  fi
  if [ "TRUE" == ${patchableDMMSServer} ]; then
    echo "installing Rhapsody Model Server patch not supported"
  fi

elif [ "ZIP" == $iFixMediaType ]; then 
  echo "using [${iFixMediaType}] media location type"
  
  if [ "FILE" == $mediaAccess ]; then
    
    if [ "TRUE" == ${patchableCLMServer} ]; then
      if [ "XX" == "X"${mediaRoot}"X" ] || [ "XX" == "X"${CLMiFixMediaRoot}"X" ] || [ "XX" == "X"${CLMiFixVersion}"X" ]; then
        errMsg="$!mediaRoot [$mediaRoot] or $!CLMiFixMediaRoot [$CLMiFixMediaRoot] or $!CLMiFixVersion [$CLMiFixVersion] not set!"
        exitWithErrMsg "$errMsg"
      else
        CLMiFixFile=${CLMiFixVersion}.zip
        CLMiFixConfigFile=${CLMiFixVersion}"_QDconfig.properties"
        iFixEngineFile="InstallMaintenanceFilesEngine.sh"
        CLMiFixZipFileFolder=${mediaRoot}"/"${CLMiFixMediaRoot}
        CLMiFixZipFileLocation=${CLMiFixZipFileFolder}"/"${CLMiFixFile}
        CLMiFixConfigFileLocation=${CLMiFixZipFileFolder}"/"${CLMiFixConfigFile}
        iFixEngineFileLocation=${CLMiFixZipFileFolder}"/"${iFixEngineFile}
        echo "CLM iFix :: $!CLMiFixZipFileFolder [${CLMiFixZipFileFolder}], $!CLMiFixZipFileLocation [${CLMiFixZipFileLocation}], $!CLMiFixConfigFileLocation [${CLMiFixConfigFileLocation}], $!iFixEngineFileLocation [${iFixEngineFileLocation}]"
      fi
    fi
    
    if [ "TRUE" == ${patchableDMServer} ]; then
      if [ "XX" == "X"${mediaRoot}"X" ] || [ "XX" == "X"${CLMiFixMediaRoot}"X" ] || [ "XX" == "X"${DMiFixMediaRoot}"X" ] || [ "XX" == "X"${DMiFixVersion}"X" ]; then
        errMsg="$!mediaRoot [$mediaRoot] or $!CLMiFixMediaRoot [$CLMiFixMediaRoot] or $!DMiFixMediaRoot [$DMiFixMediaRoot] or $!DMiFixVersion [$DMiFixVersion] not set!"
        exitWithErrMsg "$errMsg"
      else
        # engine in CLM folder, rest in DM folder
        iFixEngineFile="InstallMaintenanceFilesEngine.sh"
        CLMiFixZipFileFolder=${mediaRoot}"/"${CLMiFixMediaRoot}
        iFixEngineFileLocation=${CLMiFixZipFileFolder}"/"${iFixEngineFile}
        
        DMiFixFile=${DMiFixVersion}.zip
        DMiFixConfigFile=${DMiFixVersion}"_QDconfig.properties"
        DMiFixZipFileFolder=${mediaRoot}"/"${DMiFixMediaRoot}
        DMiFixZipFileLocation=${DMiFixZipFileFolder}"/"${DMiFixFile}
        DMiFixConfigFileLocation=${DMiFixZipFileFolder}"/"${DMiFixConfigFile}
        echo "DM iFix :: $!DMiFixZipFileFolder [${DMiFixZipFileFolder}], $!DMiFixZipFileLocation [${DMiFixZipFileLocation}], $!DMiFixConfigFileLocation [${DMiFixConfigFileLocation}], $!iFixEngineFileLocation [${iFixEngineFileLocation}]"
      fi
    fi
    
    if [ "TRUE" == ${patchableDMMSServer} ]; then
      if [ "XX" == "X"${mediaRoot}"X" ] || [ "XX" == "X"${CLMiFixMediaRoot}"X" ] || [ "XX" == "X"${DMMSiFixMediaRoot}"X" ] || [ "XX" == "X"${DMMSiFixVersion}"X" ]; then
        errMsg="$!mediaRoot [$mediaRoot] or $!CLMiFixMediaRoot [$CLMiFixMediaRoot] or $!DMMSiFixMediaRoot [$DMMSiFixMediaRoot] or $!DMMSiFixVersion [$DMMSiFixVersion] not set!"
        exitWithErrMsg "$errMsg"
      else
        # engine in CLM folder, rest in DMMS folder
        iFixEngineFile="InstallMaintenanceFilesEngine.sh"
        CLMiFixZipFileFolder=${mediaRoot}"/"${CLMiFixMediaRoot}
        iFixEngineFileLocation=${CLMiFixZipFileFolder}"/"${iFixEngineFile}
        
        DMMSiFixFile=${DMMSiFixVersion}.zip
        DMMSiFixConfigFile=${DMMSiFixVersion}"_QDconfig.properties"
        DMMSiFixZipFileFolder=${mediaRoot}"/"${DMMSiFixMediaRoot}
        DMMSiFixZipFileLocation=${DMMSiFixZipFileFolder}"/"${DMMSiFixFile}
        DMMSiFixConfigFileLocation=${DMMSiFixZipFileFolder}"/"${DMMSiFixConfigFile}
        echo "DMMS iFix :: $!DMMSiFixZipFileFolder [${DMMSiFixZipFileFolder}], $!DMMSiFixZipFileLocation [${DMMSiFixZipFileLocation}], $!DMMSiFixConfigFileLocation [${DMMSiFixConfigFileLocation}], $!iFixEngineFileLocation [${iFixEngineFileLocation}]"
      fi
    fi
    
    # NFS enabling is common to all patches
    if [ "X"$nfsEnabled"X" == "XX" ]; then
      echo "no NFS drive exists, assuming any ZIP files are local"
      nfsEnabled=FALSE
    else
      # need to check value
      if [ "FALSE" == $nfsEnabled ]; then
        echo "no NFS drive exists, assuming any ZIP files are local"
      elif [ "TRUE" == $nfsEnabled ]; then 
        echo "NFS drive exists, assuming any ZIP files are remote"
      else
        errMsg="["${nfsEnabled}"] nfsEnabled not supported"
        exitWithErrMsg "$errMsg"
      fi
    fi
      
  else
    errMsg="$!mediaAccess [$mediaAccess] not supported!"
    exitWithErrMsg "$errMsg"
  fi

else
  errMsg="["${iFixMediaType}"] media location type not supported"
  exitWithErrMsg "$errMsg"
fi



## =======================================================
## COMMON FOLDERS ##
## ===========
# as all input checks complete its now possible to create the download folders to contain the unzip parts
echo "delete any old iFix content and empty folders for the new content"
deleteFolder ${localIFixHeadFolder}
makeFolderFullPath ${localIFixHeadFolder}
makeFolderFullPath ${localIFixZipFolder}
makeFolderFullPath ${localIFixZipFolderFiles}
# ensure folder used by the engine to unzip the patch into is writable
chmod 777 ${localIFixZipFolderFiles}

# may well require extra folders
if [ "CLM" == $iFixMediaType ]; then
  if [ "TRUE" == ${patchableCLMServer} ]; then
    echo "extra folders required for CLM when [iFixMediaType = $iFixMediaType]"
    makeFolderFullPath ${CLMlocalIFixDownloadedFilesFolder}
    makeFolderFullPath ${CLMlocalIFixDownloadedAdditionalFilesFolder}
    makeFolderFullPath ${CLMlocalIFixDownloadedTempFolder}
  fi
fi
if [ "ZIP" == $iFixMediaType ]; then
  echo "no extra folders required for [iFixMediaType = $iFixMediaType]"
fi


## =======================================================
## DOWNLOAD / COPY media##
## =====================


if [ "CLM" == $iFixMediaType ]; then 

  if [ "TRUE" == ${patchableCLMServer} ]; then
    echo "== get CLM pipeline patch parts  =="
    # - set maintClmStreamID using CLMsetMaintStreamID
    CLMsetMaintStreamID $appRelease
    
    # - download full contents of the iFix 
  
    ## construct location of patch zipfile
    CLMpatchZipFileName="CLM_server_patch_"${maintClmStreamID}"-"${CLM_BUILD_ID}".zip"
    CLMpatchZipFolder=${CLM_BUILDROOTURL}"/"${clmPatchZipSuffix}
    CLMpatchZipFile=${CLMpatchZipFolder}"/"${CLMpatchZipFileName}
    echo "CLM patch zip file (url) ["${CLMpatchZipFile}"]"
  
    ## download patch ZIP file direct to  ${CLMlocalIFixDownloadedFilesFolder}
    echo "==== downloading iFix patch zip file ===="
    timestamp=`date +%s`
    echo "DEBUG (ready to get ZIP) : current epoch time is ["${timestamp}"] Sec"
    cd ${CLMlocalIFixDownloadedFilesFolder}
    downloadHttpZip ${CLMpatchZipFile}
    echo "current contents of [${CLMlocalIFixDownloadedFilesFolder}]"
    ls -la
  
    ## construct location of additional files zipfile
    CLMadditionalZipFileName="AdditionalFiles_"${maintClmStreamID}"-"${CLM_BUILD_ID}".zip"
    CLMadditionalZipFolder=${CLM_BUILDROOTURL}"/"${clmPatchZipSuffix}
    CLMadditionalZipFile=${CLMpatchZipFolder}"/"${CLMadditionalZipFileName}
    echo "CLM additional zip file (url) ["${CLMadditionalZipFile}"]"
    
    ## download additional ZIP file to ${CLMlocalIFixDownloadedAdditionalFilesFolder}
    echo "==== downloading iFix additional zip file ===="
    timestamp=`date +%s`
    echo "DEBUG (ready to get ZIP) : current epoch time is ["${timestamp}"] Sec"
    cd ${CLMlocalIFixDownloadedAdditionalFilesFolder}
    downloadHttpZip ${CLMadditionalZipFile}
    echo "current contents of [${CLMlocalIFixDownloadedAdditionalFilesFolder}]"
    ls -la
    
    timestamp=`date +%s`
    echo "DEBUG (ready to unzip ZIP) : current epoch time is ["${timestamp}"] Sec"
    
    # extract the contents of the additional zip file
    echo "==== extracting contents of iFix additional zip file to temp folder ===="
    unzipZip $CLMadditionalZipFileName ${CLMlocalIFixDownloadedTempFolder}
    echo "current contents of [${CLMlocalIFixDownloadedTempFolder}]"
    ls -la
    
    # process the "temp files"
    echo "==== processing the contents of the temp folder ===="
    cd ${CLMlocalIFixDownloadedTempFolder}
    tempString=`ls`
    read -r -a tempArray <<< ${tempString}
    for item in "${tempArray[@]}"
    do
      echo "item = [$item]"
      case "$item" in
        *.zip )
          echo "it is a zip file"
          # does it contain just a single war file?
          fileCount=`unzip -Z -1 ${item} | wc -w`
          warCount=`unzip -Z -1 ${item} | grep war | wc -w`
          if [ $fileCount -eq 1 ] && [ $warCount -eq 1 ]; then
            echo "fileCount [${fileCount}] warCount [${warCount}]"
            tempWarFile=`unzip -Z -1 ${item} | grep war`
            echo "containing only a single war file, so extract the file [$tempWarFile]"
            unzipZipPartial ${item} ${CLMlocalIFixDownloadedFilesFolder} $tempWarFile
          else
            echo "containing more than just a single war file, so just copy the zip [$item]"
            copyFile $item ${CLMlocalIFixDownloadedFilesFolder}
          fi
          ;;
        *)
          echo "it is NOT a zip file, so just copy the file [$item]"
          copyFile $item ${CLMlocalIFixDownloadedFilesFolder}
          ;;
      esac
    done
    
    cd ${CLMlocalIFixDownloadedFilesFolder}
    echo "current contents of [${CLMlocalIFixDownloadedFilesFolder}]"
    ls -la
    
    timestamp=`date +%s`
    echo "DEBUG (ZIP unzipped) : current epoch time is ["${timestamp}"] Sec"
    
    # now create a zip from the bits and place it in ${localIFixZipFolder}
    echo "==== creating the iFix zipfile ===="
    CLMiFixFile="assembledCLM"${iFixVersion}.zip
    CLMiFixFilePath=${localIFixZipFolder}"/"$CLMiFixFile
    zipFolder $CLMiFixFile ${localIFixZipFolder}
    checkFileExists $CLMiFixFilePath
    echo "listing content of the zip file [$CLMiFixFilePath]"
    listZipContents $CLMiFixFilePath
  fi
  
  if [ "TRUE" == ${patchableDMServer} ]; then
    echo "== DM pipeline not supported  =="
  fi
  if [ "TRUE" == ${patchableDMMSServer} ]; then
    echo "== DM pipeline not supported  =="
  fi
  
fi


if [ "ZIP" == $iFixMediaType ]; then
  ## set paths to engine, zip and config files
  
  # engine is common to all 
  if [ "TRUE" == ${patchableCLMServer} ] || [ "TRUE" == ${patchableDMServer} ] || [ "TRUE" == ${patchableDMMSServer} ]; then
    timestamp=`date +%s`
    echo "DEBUG (ready to download the iFix engine) : current epoch time is ["${timestamp}"] Sec"
    # only download a copy if NFS has been setup
    if [ "TRUE" == ${nfsEnabled} ]; then
      # copy the zip
      cd ${localIFixZipFolder}
      # use copyZip to copy the engine
      copyZip ${iFixEngineFileLocation}
      ls -la
      iFixEngineFilePath=${localIFixZipFolder}"/"${iFixEngineFile}
      zipNeedsDeleted=TRUE
    else
      # just update the path to the engine
      iFixEngineFilePath=${iFixEngineFileLocation}
    fi
    timestamp=`date +%s`
    echo "DEBUG (engine available) : current epoch time is ["${timestamp}"] Sec"
  fi
  
  # if required get the CLM zip and config file
  if [ "TRUE" == ${patchableCLMServer} ]; then
    timestamp=`date +%s`
    echo "DEBUG (ready to get CLM ZIP) : current epoch time is ["${timestamp}"] Sec"
    # only download a copy if NFS has been setup
    if [ "TRUE" == ${nfsEnabled} ]; then
      # copy the zip
      cd ${localIFixZipFolder}
      # use copyZip to copy the config and zip files
      copyZip ${CLMiFixZipFileLocation}
      copyZip ${CLMiFixConfigFileLocation}
      ls -la
      CLMiFixFilePath=${localIFixZipFolder}"/"${CLMiFixFile}
      CLMiFixConfigFilePath=${localIFixZipFolder}"/"${CLMiFixConfigFile}
      zipNeedsDeleted=TRUE
    else
      # copy the zip, so its permissions can be changed
      cd ${localIFixZipFolder}
      copyZip ${CLMiFixZipFileLocation}
      ls -la
      CLMiFixFilePath=${localIFixZipFolder}"/"${CLMiFixFile}
      # will use the other files direct from their "local locations"
      CLMiFixConfigFilePath=${CLMiFixConfigFileLocation}
    fi
    timestamp=`date +%s`
    echo "DEBUG (CLM zip file available) : current epoch time is ["${timestamp}"] Sec"
    echo "==== listing content of the CLM zip file [$CLMiFixFilePath] ===="
    listZipContents $CLMiFixFilePath
  fi
  
  # if required get the DM zip and config file
  if [ "TRUE" == ${patchableDMServer} ]; then
    timestamp=`date +%s`
    echo "DEBUG (ready to get DM ZIP) : current epoch time is ["${timestamp}"] Sec"
    # only download a copy if NFS has been setup
    if [ "TRUE" == ${nfsEnabled} ]; then
      # copy the zip
      cd ${localIFixZipFolder}
      # use copyZip to copy the config and zip files
      copyZip ${DMiFixZipFileLocation}
      copyZip ${DMiFixConfigFileLocation}
      ls -la
      DMiFixFilePath=${localIFixZipFolder}"/"${DMiFixFile}
      DMiFixConfigFilePath=${localIFixZipFolder}"/"${DMiFixConfigFile}
      zipNeedsDeleted=TRUE
    else
      # copy the zip, so its permissions can be changed
      cd ${localIFixZipFolder}
      copyZip ${DMiFixZipFileLocation}
      ls -la
      DMiFixFilePath=${localIFixZipFolder}"/"${DMiFixFile}
      # will use the other files direct from their "local locations"
      DMiFixConfigFilePath=${DMiFixConfigFileLocation}
    fi
    timestamp=`date +%s`
    echo "DEBUG (DM zip file available) : current epoch time is ["${timestamp}"] Sec"
    echo "==== listing content of the DM zip file [$DMiFixFilePath] ===="
    listZipContents $DMiFixFilePath
  fi
  
  # if required get the Rhapsody Model server zip and config file
  if [ "TRUE" == ${patchableDMMSServer} ]; then
    timestamp=`date +%s`
    echo "DEBUG (ready to get DM ZIP) : current epoch time is ["${timestamp}"] Sec"
    # only download a copy if NFS has been setup
    if [ "TRUE" == ${nfsEnabled} ]; then
      # copy the zip
      cd ${localIFixZipFolder}
      # use copyZip to copy the config and zip files
      copyZip ${DMMSiFixZipFileLocation}
      copyZip ${DMMSiFixConfigFileLocation}
      ls -la
      DMMSiFixFilePath=${localIFixZipFolder}"/"${DMMSiFixFile}
      DMMSiFixConfigFilePath=${localIFixZipFolder}"/"${DMMSiFixConfigFile}
      zipNeedsDeleted=TRUE
    else
      # copy the zip, so its permissions can be changed
      cd ${localIFixZipFolder}
      copyZip ${DMMSiFixZipFileLocation}
      ls -la
      DMMSiFixFilePath=${localIFixZipFolder}"/"${DMMSiFixFile}
      # will use the other files direct from their "local locations"
      DMMSiFixConfigFilePath=${DMMSiFixConfigFileLocation}
    fi
    timestamp=`date +%s`
    echo "DEBUG (DMMS zip file available) : current epoch time is ["${timestamp}"] Sec"
    echo "==== listing content of the Rhapsody Model Server zip file [$DMMSiFixFilePath] ===="
    listZipContents $DMMSiFixFilePath
  fi
  
fi


if [ "ZIP" == $iFixMediaType ]; then
  echo "use media server version of the install engine [$iFixEngineFilePath]"
  
  if [ "TRUE" == ${patchableCLMServer} ]; then
    echo "== installing CLM patch  =="
    echo "use media server version of the config file [$CLMiFixConfigFilePath]"
    echo "use media server patch zip [$CLMiFixFilePath]"
  else
    echo "== not installing CLM patch  =="
  fi
  if [ "TRUE" == ${patchableDMServer} ]; then
    echo "== installing DM patch  =="
    echo "use media server version of the config file [$DMiFixConfigFilePath]"
    echo "use media server patch zip [$DMiFixFilePath]"
  else
    echo "== not installing DM patch  =="
  fi
  if [ "TRUE" == ${patchableDMMSServer} ]; then
    echo "== installing Rhapsody Model Server patch  =="
    echo "use media server version of the config file [$DMMSiFixConfigFilePath]"
    echo "use media server patch zip [$DMMSiFixFilePath]"
  else
    echo "== not installing Rhapsody Model Server patch  =="
  fi
fi

if [ "CLM" == $iFixMediaType ]; then
  
  if [ "TRUE" == ${patchableCLMServer} ]; then
    echo "== installing CLM patch  =="
    CLMiFixQdConfigFile="QDiFixConfig.properties"
    CLMiFixConfigFilePath=${workingDir}"/"$CLMiFixQdConfigFile
    iFixEngineFilePath=${workingDir}"/InstallMaintenanceFilesEngine.sh"
    echo "use latest UCD version of the install engine [$iFixEngineFilePath]"
    echo "use latest UCD version of the config file [$CLMiFixConfigFilePath]"
    echo "use latest UCD assembled patch zip [$CLMiFixFilePath]"
  else
    echo "== not installing CLM patch  =="
  fi
  if [ "TRUE" == ${patchableDMServer} ]; then
    errMsg="installing DM patch not supported for this mode [$iFixMediaType]"
    exitWithErrMsg "$errMsg"
  else
    echo "== not installing DM patch  =="
  fi
  if [ "TRUE" == ${patchableDMMSServer} ]; then
    errMsg="installing Rhapsody Model Server patch not supported for this mode [$iFixMediaType]"
    exitWithErrMsg "$errMsg"
  else
    echo "== not installing Rhapsody Model Server patch  =="
  fi

fi


####
######
########

##echo "leaving prior to any install engine running"
##echo " -- may need to delete [${localIFixHeadFolder}]"
##exit 0




## =======================================================
## MAKE THE IFIX ZIPs VISIBLE
## THEN INSTALL THE IFIXes
## ===========================
# to ensure correct file ownership for any new file use the existing user of ${CLM_HOME}"/server"
folderToGetAttriburesFrom=${CLM_HOME}"/server"
checkFolderExists $folderToGetAttriburesFrom
tempUser=`ls -ld $folderToGetAttriburesFrom | cut -d' ' -f 3`
#tempGroup=`ls -ld ${folderToGetAttriburesFrom} | cut -d' ' -f 4`
#tempPermissions=`stat -c "%a" ${folderToGetAttriburesFrom}`  

echo "--- current ownership of [$folderToGetAttriburesFrom] is [$tempUser]--- "

if [ "TRUE" == ${patchableCLMServer} ]; then
  echo "make the CLM iFix zip file visible to [$tempUser]"
  chmod -R 666 $CLMiFixFilePath
  ls -la $CLMiFixFilePath
  
  echo "--- install the CLM iFix as user [$tempUser]--- "
  cd $workingDir
  su -c "sh $iFixEngineFilePath $CLMiFixConfigFilePath $CLMiFixFilePath" -- $tempUser
  if [ $? -ne 0 ]; then
    errMsg="--++ failed to install CLM iFix"
    exitWithErrMsg "$errMsg"
  fi
fi

if [ "TRUE" == ${patchableDMServer} ]; then
  echo "make the DM iFix zip file visible to [$tempUser]"
  chmod -R 666 $DMiFixFilePath
  ls -la $DMiFixFilePath
  
  echo "--- install the DM iFix as user [$tempUser]--- "
  cd $workingDir
  su -c "sh $iFixEngineFilePath $DMiFixConfigFilePath $DMiFixFilePath" -- $tempUser
  if [ $? -ne 0 ]; then
    errMsg="--++ failed to install DM iFix"
    exitWithErrMsg "$errMsg"
  fi
fi
if [ "TRUE" == ${patchableDMMSServer} ]; then
  echo "make the Rhapsody Model Server iFix zip file visible to [$tempUser]"
  chmod -R 666 $DMMSiFixFilePath
  ls -la $DMMSiFixFilePath
  
  echo "--- install the Rhapsody Model Server iFix as user [$tempUser]--- "
  cd $workingDir
  su -c "sh $iFixEngineFilePath $DMMSiFixConfigFilePath $DMMSiFixFilePath" -- $tempUser
  if [ $? -ne 0 ]; then
    errMsg="--++ failed to install Rhapsody Model Server iFix"
    exitWithErrMsg "$errMsg"
  fi
fi


## =======================================================
## TIDY UP ##
## ==========
echo "delete all iFix downloads folders"
deleteFolder ${localIFixHeadFolder}



## =======================================================
## === finished ====
errMsg="reached the end of the script"
exitWithOkMsg "$errMsg"

