#!/bin/sh
#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2015, 2016. All Rights Reserved.
#
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************
#
# Name: installDB2server.sh
#
# Installs DB2 server
#   Execute the program: sh .\installDB2server.sh
#
#********************************************************************************

# expect UCD to supply these scripts in the same directory as <installDB2server.sh>
# ---------------------------------------------------------------------------------
# getUnixBit.sh
# testEntry.sh
# checkUser.sh


#############
## FUNCTIONS 
#############


exitWithErrMsg()
{
  tempString=$1
  echo "-- ERROR [${scriptName}] --"
  echo "-- message: $tempString"
  exit 1
}


## usage checkFileExists tempFile
checkFileExists()
{
  tempFile=$1
  if [ -f $tempFile ]; then
    echo "file [$tempFile] located"
  else
    echo "ERROR: [$tempFile] does not exist"
    exit 1
  fi
}


downloadHttpZip()
{
  fileUrl=$1
  filename=$(basename $fileUrl)

  sh $AUTO_PREP/MediaParameters/doWget.sh $fileUrl
  if [ $? == 0 ]
  then
    echo "download complete"
    checkFileExists $filename
  else
    errMsg="failed to download file ["${filename}"]"
    exitWithErrMsg "$errMsg"
  fi
}

copyZip()
{
  file=$1
  filename=$(basename $file)
  echo "copy ["${file}"] to ["${filename}"]"
  cp $file $filename
  if [ $? == 0 ]
  then
    echo "copy complete" 
    checkFileExists $filename
  else
    errMsg="failed to copy file ["${filename}"]"
    exitWithErrMsg "$errMsg"
  fi
}


unzipTar()
{
  tarFile=$1
  targetDir=$2
  
  pwd
  nowDir=`pwd -P`
  cd $targetDir
  echo "uncompressing the tar file to.."
  pwd
  cat ${tarFile} | gzip -d | tar -xvf -
  if [ $? == 0 ]; then
    echo "tar explosion successful" 
  else
    errMsg="failed to explode ["${tarFile}"]"
    exitWithErrMsg "$errMsg"
  fi
  cd $nowDir
  pwd
  
}


addGroup()
{
  groupToAdd=$1
  
  echo "add new group [$groupToAdd]"
  groupadd $groupToAdd
  if [ $? == 0 ]; then
    echo "group add successful"
  else
    errMsg="failed to add group"
    exitWithErrMsg "$errMsg"
  fi
}


addUserToGroups()
{
  userToAdd=$1
  primaryGroup=$2
  if [ $# -eq 2 ]; then
    echo "add new user [$userToAdd] to P[$primaryGroup]"
    addUserToGroupsCommand="useradd -m -d /home/$userToAdd -g $primaryGroup $userToAdd"
  else
    secondaryGroup=$3
    echo "add new user [$userToAdd] to P[$primaryGroup] S[$secondaryGroup]"
    addUserToGroupsCommand="useradd -m -d /home/$userToAdd -g $primaryGroup -G $secondaryGroup $userToAdd"
  fi
  
  # run the selected command
  $addUserToGroupsCommand
  if [ $? == 0 ]; then
    echo "add user to group(s) successful"
  else
    errMsg="failed to add user to group(s)"
    exitWithErrMsg "$errMsg"
  fi
}

checkDBExist()
{
    su - $ibmQseInst_User -c "db2ls -c"
    if [ $? -eq 0 ];then
       echo "   DB2 is currently installed on the target machine"
       return 0
    else
       echo "   DB2 is not currently installed on the target machine, proceed with install process"
       return 1
    fi
}

checkOS()
{
	####################################
	# may need to check OS 
	bRH6flag=FALSE
	
	RHversion=`cat /etc/redhat-release`
	echo "Current Red Hat Release $RHversion"
	
	echo $RHversion | grep "Santiago"
	if [ $? -eq 0 ]; then
	  echo "OS is Red Hat 6"
	  bRH6flag=TRUE
	fi
}
####################
##                ##
## ==== MAIN ==== ##
##                ##
####################

timestamp=`date +%s`
echo "DEBUG (start) : current epoch time is ["$timestamp"] Sec"



# set up local environment for this script
scriptName=$0
workingDir=`pwd -P`
envProp=/etc/env.properties
. $envProp

# get credentials
userCreds=$AUTO_PREP/UserParameters/userCreds.properties
. $userCreds

# Get ibmQseInst_PW Password for the setup
# ======================================
tempibmQseInst_PW=`echo ${ibmQseInst_PW} | base64 -d`

# get media properties
mediaProp=$AUTO_PREP/MediaParameters/media.properties
. $mediaProp

if checkDBExist; then 
   echo "Database is installed on the system, exit installation step"
   exit 0
fi

if [ "ZIP" == $db2MediaType ] || [ "FILE" == $db2MediaType ]; then 
  echo "using [${db2MediaType}] media location type"
else
  errMsg="["${db2MediaType}"] media location type not supported"
  exitWithErrMsg "$errMsg"
fi


if [ "X"$nfsEnabled"X" == "XX" ]; then
  echo "no NFS drive exists, assuming any ZIP files are local"
  nfsEnabled=FALSE
else
  # need to check value
  if [ "FALSE" == $nfsEnabled ]; then
    echo "no NFS drive exists, assuming any ZIP files are local"
    nfsEnabled=FALSE
  elif [ "TRUE" == $nfsEnabled ]; then 
    echo "NFS drive exists, assuming any ZIP files are remote"
  else
    errMsg="["${nfsEnabled}"] nfsEnabled not supported"
    exitWithErrMsg "$errMsg"
  fi
fi

if [ "X"$createMediaOnly"X" == "XX" ]; then
  echo "installing DB2"
  createMediaOnly=FALSE
else
  # need to check value
  if [ "FALSE" == $createMediaOnly ]; then
    echo "installing DB2"
    createMediaOnly=FALSE
  elif [ "TRUE" == $createMediaOnly ]; then 
    echo "adding DB2 installer to media server"
  else
    errMsg="["${createMediaOnly}"] createMediaOnlyFlag not supported"
    exitWithErrMsg "$errMsg"
  fi
fi


if [ "X"$debugDoNotInstall"X" == "XX" ]; then
  echo "installing DB2"
  debugDoNotInstall=FALSE
else
  # need to check value
  if [ "FALSE" == $debugDoNotInstall ]; then
    echo "installing DB2"
    debugDoNotInstall=FALSE
  elif [ "TRUE" == $debugDoNotInstall ]; then
    echo "DEBUG: will exit prior to installing DB2"
  else
    errMsg="["${debugDoNotInstall}"] debugDoNotInstallFlag not supported"
    exitWithErrMsg "$errMsg"
  fi
fi


if [ "FALSE" == $createMediaOnly ]; then
  # Check DB2 server existence, we should not install DB2 if it is already setup
  # ============================================================================
  
  id $ibmQseInst_User
  if [ $? -eq 0 ]; then 
    # user already exists, so DB2 may already be installed
    # list environment settings is an easy test to do
    su - $ibmQseInst_User -c "db2set -all"
    # process su command response
    if [ $? -eq 0 ]; then
      echo "DB2 already exists (by QD), exit the installation process"
      exit 1
    else
      echo "DB2 is not currently installed (by QD) on the target machine, starting install process"
    fi
  else
    echo "DB2 user ["$ibmQseInst_User"] does not exist, so assume DB2 not previously installed, by QD"
  fi
fi


###################################################

# get system bits
# ===============
# Gss7 DB server returns Linux64
BITS=`sh ./getUnixBit.sh`
echo "OS bits [$BITS]"

## ======================================================================
## set $db2InstallFile location
unzipFolderNeedsDeleted=FALSE
zipNeedsDeleted=FALSE
if [ "FILE" == $db2MediaType ]; then 
  ## using file ${mediaRoot}/${db2MediaRoot}/${db2File}
  if [ "XX" == "X"${mediaRoot}"X" ] || [ "XX" == "X"${db2MediaRoot}"X" ] || [ "XX" == "X"${db2File}"X" ]; then
    errMsg="$!mediaRoot [$mediaRoot] or $!db2MediaRoot [$db2MediaRoot] or $!db2File [$db2File] not set!"
    exitWithErrMsg "$errMsg"
  else
    db2InstallFolder=${mediaRoot}"/"${db2MediaRoot}
    db2InstallFile=${db2File}
    db2InstallFileLocation=${db2InstallFolder}"/"${db2InstallFile}
    echo "using file $!db2InstallFileLocation [$db2InstallFileLocation]"
    checkFileExists ${db2InstallFileLocation}
  fi
elif [ "ZIP" == $db2MediaType ]; then
  # need to calculate $db2ZipFileLocation
  if [ "FILE" == $mediaAccess ]; then
    ## using file ${mediaRoot}/${db2MediaRoot}/${db2File}
    if [ "XX" == "X"${mediaRoot}"X" ] || [ "XX" == "X"${db2MediaRoot}"X" ] || [ "XX" == "X"${db2Version}"X" ] || [ "XX" == "X"${db2File}"X" ]; then
      errMsg="$!mediaRoot [$mediaRoot] or $!db2MediaRoot [$db2MediaRoot] or $!db2Version [$db2Version] or $!db2File [$db2File] not set!"
      exitWithErrMsg "$errMsg"
    else
      db2ZipFileLocation=${mediaRoot}"/"${db2MediaRoot}"/"${db2Version}"/"${db2File}
      checkFileExists ${db2ZipFileLocation}
    fi
  fi
  if [ "HTTP" == $mediaAccess ] || [ "HTTPS" == $mediaAccess ]; then
    if [ "X"$db2File"X" == "XX" ]; then
      tempStr="notURL"
    else
      tempStr=`echo ${db2File} | cut -c -4`
    fi
    if [ "http" == $tempStr ]; then
      ## $db2File conatins the full URL
      if [ "XX" == "X"${db2File}"X" ]; then
        errMsg="$!db2File [$db2File] not set!"
        exitWithErrMsg "$errMsg"
      else
        db2ZipFileLocation=${db2File}
      fi
    else
      # calculate the URL = ${mediaServer}/${db2MediaRoot}/${db2Version}/${db2File}
      if [ "XX" == "X"${mediaServer}"X" ] || [ "XX" == "X"${db2MediaRoot}"X" ] || [ "XX" == "X"${db2Version}"X" ] || [ "XX" == "X"${db2File}"X" ]; then
        errMsg="$!mediaServer [$mediaServer] or $!db2MediaRoot [$db2MediaRoot] or $!db2Version [$db2Version] not set! or $!db2File [$db2File] not set!"
        exitWithErrMsg "$errMsg"
      else
        db2ZipFileLocation=${mediaServer}"/"${db2MediaRoot}"/"${db2Version}"/"${db2File}
      fi
    fi
  fi  

  ## using zipped up file
  if [ "X"$db2ZipFileLocation"X" == "XX" ]; then
    errMsg="$!db2ZipFileLocation [$db2ZipFileLocation] not set!"
    exitWithErrMsg "$errMsg"
  else
    echo "using zip file, $!db2ZipFileLocation [$db2ZipFileLocation]"
    zipFileName=$(basename $db2ZipFileLocation)
    
    ## download the zip (http) OR copy the zip (nfs)
    #downloadDir=${mediaRoot}"/"${db2MediaRoot}
    downloadDir=${QD_InternalMedia}"/"${db2MediaRoot}
    unzippedDir=${downloadDir}"/unzipped"
    mkdir -p ${downloadDir}
    mkdir -p ${unzippedDir}
    cd ${downloadDir}
    tempStr=`echo ${db2ZipFileLocation} | cut -c -4`
    timestamp=`date +%s`
    echo "DEBUG (ready to get TAR) : current epoch time is ["$timestamp"] Sec"
    if [ "http" == $tempStr ]; then
      ## download the zip from the url
      downloadHttpZip $db2ZipFileLocation
      zipNeedsDeleted=TRUE
    else
      # only download a copy if NFS has been setup
      if [ "TRUE" == ${nfsEnabled} ]; then
        # copy the zip
        copyZip $db2ZipFileLocation
        zipNeedsDeleted=TRUE
      fi
    fi
    timestamp=`date +%s`
    echo "DEBUG (ready to unzip TAR) : current epoch time is ["$timestamp"] Sec"
    ## unzip
    if [ "http" == $tempStr ]; then
      unzipTar ${downloadDir}"/"${zipFileName} ${unzippedDir}
      unzipFolderNeedsDeleted=TRUE
    else
      if [ "TRUE" == ${nfsEnabled} ]; then
        unzipTar ${downloadDir}"/"${zipFileName} ${unzippedDir}
        unzipFolderNeedsDeleted=TRUE
      else
        unzipTar $db2ZipFileLocation ${unzippedDir}
        unzipFolderNeedsDeleted=TRUE
      fi
    fi
    timestamp=`date +%s`
    echo "DEBUG (TAR exploded) : current epoch time is ["$timestamp"] Sec"
    
    db2InstallFileLocation=${unzippedDir}"/"${db2FolderName}"/db2setup"
    echo "using unzipped file, $!db2InstallFileLocation [$db2InstallFileLocation]"
    checkFileExists ${db2InstallFileLocation}
  fi
fi


## =============================================================
## === exit if only creating a media server

if [ "TRUE" == $createMediaOnly ]; then
  echo "media server update done"
  exit 0
fi


## =============================================================
## === DEBUG early exit

if [ "TRUE" == $debugDoNotInstall ]; then
  echo "DEBUG: Exiting prior to installing DB2"
  exit 0
fi


# Prepare DB2 Users
# =================
# - check and update users [DAS User, Instance User and Fence User]
cd $workingDir
echo "initial test on [$ibmQseDAS_User]"
sh ./testEntry.sh $ibmQseDAS_User
echo "initial test on [$ibmQseInst_User]"
sh ./testEntry.sh $ibmQseInst_User
echo "initial test on [$ibmQseFence_User]"
sh ./testEntry.sh $ibmQseFence_User

# - Check user's home directory 
echo "check [$ibmQseDAS_User] for valid home dir"
sh ./checkUser.sh $ibmQseDAS_User
echo "check [$ibmQseInst_User] for valid home dir"
sh ./checkUser.sh $ibmQseInst_User
echo "check [$ibmQseFence_User] for valid home dir"
sh ./checkUser.sh $ibmQseFence_User

# - Add Groups
addGroup $ibmQseDAS_Group
addGroup $ibmQseInst_Group
addGroup $ibmQseFence_Group

# - Add Users and assign groups
# only a primary group
addUserToGroups $ibmQseDAS_User $ibmQseDAS_Group
addUserToGroups $ibmQseFence_User $ibmQseFence_Group
# primary and secondary groups
addUserToGroups $ibmQseInst_User $ibmQseInst_Group $ibmQseDAS_Group

# - list current state
echo "list /home"
ls -la /home
id $ibmQseDAS_User
id $ibmQseFence_User
id $ibmQseInst_User


# - Security Compliance on linux  
echo "For security compliance, prevent login on Linux platforms"
echo "use su - user as root to use accounts instead of logging in"
passwd -l $ibmQseDAS_User
passwd -l $ibmQseFence_User

echo $tempibmQseInst_PW | passwd $ibmQseInst_User --stdin
# Set password aging to min 30 days, max 90 and 10 day warning
passwd $ibmQseInst_User -n 30 -x 90 -w 10


echo "install DB2 [$db2Version] using [$db2InstallFileLocation]"

# verify the install target is specified
if [ "X"$db2RootDirUnix"X" == "XX" ]; then
  errMsg="$!db2RootDirUnix [$db2RootDirUnix] not set!"
  exitWithErrMsg "$errMsg"
else
  installTarget=${db2RootDirUnix}"/"${db2Version}"/server"
  echo "*** Installing DB2 to [$installTarget]"
fi

# setup response file
reponseTemplate=$workingDir/db2server_template.rsp 
responseFile=$workingDir/db2server.rsp
cp $reponseTemplate $responseFile
checkFileExists ${responseFile}

echo "*** update response file [$responseFile]"
echo "*** set install target folder [$installTarget]"
sed -i s%_DB2_INSTALL_FOLDER_%$installTarget% $responseFile

# Tivoli System Automation for Multiplatforms (TSAMP)
if [ "TRUE" == $installDB2TSAMP ]; then
  echo "*** install TSAMP"
  sed -i s%_DB2_TSAMP_COMP_%COMP% $responseFile
else
  echo "*** do not install TSAMP"
  sed -i s%_DB2_TSAMP_COMP_%\*COMP% $responseFile
fi
installCommand="$db2InstallFileLocation -r $workingDir/db2server.rsp -l $workingDir/db2setup.log"

echo "*** install command [$installCommand]"
echo "*** installing...."
$installCommand
if [ $? == 0 ]
then
  echo "*** install successful"
else
  errMsg="failed to install DB2, Installation Aborted...."
  exitWithErrMsg "$errMsg"
fi

# what have you installed?
echo "checking...."
$installTarget/install/db2ls -q -p -b $installTarget
if [ $? == 0 ]
then
  echo "check successful"
  #######
  #Add start up scripts to /etc/init.d
  sed -i s%_INSTANCE_OWNER_%$ibmQseInst_User% $workingDir/ibm-db2
  checkOS
  if [ "TRUE" == $bRH6flag ]; then
  	cp $workingDir/ibm-db2 /etc/init.d
    chmod o+x /etc/init.d/ibm-db2
    chkconfig --add ibm-db2
    echo "checking start up scripts"
    ls -l /etc/init.d/ibm-db2
  else
     if [ ! -d /usr/lib/systemd/scripts/ ];then
        mkdir /usr/lib/systemd/scripts/
     fi
     cp $workingDir/ibm-db2 /usr/lib/systemd/scripts/
     chmod o+x /usr/lib/systemd/scripts/ibm-db2
     cp $workingDir/ibm-db2.service /usr/lib/systemd/system/
     systemctl enable ibm-db2.service
     echo "checking start up scripts"
     ls -l /usr/lib/systemd/scripts/ibm-db2
     ls -la /etc/systemd/system/multi-user.target.wants | grep ibm-db2.service
  fi

else
  errMsg="failed to list DB2, Installation Aborted...."
  exitWithErrMsg "$errMsg"
fi

# - wait 30 secs
sleep 30

# tidy up by deleting anything downloaded or unzipped
# ===================================================
if [ "TRUE" == $unzipFolderNeedsDeleted ]; then
  echo "Removing the directory ["${unzippedDir}"]"
  rm -Rf ${unzippedDir}
fi
if [ "TRUE" == $zipNeedsDeleted ]; then
  echo "Removing the directory ["${downloadDir}"]"
  rm -Rf ${downloadDir}
fi


## =======================================================
## === finished ====
timestamp=`date +%s`
echo "DEBUG (EOF) : current epoch time is ["$timestamp"] Sec"
echo "SUCCESS: installation complete"
exit 0
