#!/bin/sh
#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2016. All Rights Reserved.
#
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************

#############
## FUNCTIONS 
#############


exitWithErrMsg()
{
  tempString=$1
  echo "-- ERROR [${scriptName}] --"
  echo "-- message: $tempString"
  exit 1
}


## usage checkFileExists tempFile
checkFileExists()
{
  tempFile=$1
  if [ -f $tempFile ]; then
    echo "file [$tempFile] located"
  else
    echo "ERROR: [$tempFile] does not exist"
    exit 1
  fi
}


downloadHttpZip()
{
  fileUrl=$1
  filename=$(basename $fileUrl)

  sh $AUTO_PREP/MediaParameters/doWget.sh $fileUrl
  if [ $? == 0 ]
  then
    echo "download complete"
    checkFileExists $filename
  else
    errMsg="failed to download file ["${filename}"]"
    exitWithErrMsg "$errMsg"
  fi
}

copyZip()
{
  file=$1
  filename=$(basename $file)
  echo "copy ["${file}"] to ["${filename}"]"
  cp $file $filename
  if [ $? == 0 ]
  then
    echo "copy complete" 
    checkFileExists $filename
  else
    errMsg="failed to copy file ["${filename}"]"
    exitWithErrMsg "$errMsg"
  fi
}


unzipZip()
{
  zipFile=$1
  targetDir=$2
  
  unzip ${zipFile} -d ${targetDir}
  if [ $? == 0 ]; then
    echo "unzip successful" 
  else
    errMsg="failed to unzip  ["${zipFile}"]"
    exitWithErrMsg "$errMsg"
  fi
}

unzipOracleZipFiles()
{
  oracleNameHead=$1
  oracleDiskCount=$2
  unzippedDir=$3

  for (( c=1; c<=${oracleDiskCount}; c++ )); do
    unzipZip $oracleRepoDir/${oracleNameHead}_${c}of${oracleDiskCount}.zip ${unzippedDir}
  done
}


getOracleZipFiles()
{
  urlBase=$1
  oracleNameHead=$2
  oracleDiskCount=$3

  for (( c=1; c<=${oracleDiskCount}; c++ )); do
    downloadHttpZip ${urlBase}/${oracleNameHead}_${c}of${oracleDiskCount}.zip
  done
}

copyOracleZipFiles()
{
  fileBase=$1
  oracleNameHead=$2
  oracleDiskCount=$3

  for (( c=1; c<=${oracleDiskCount}; c++ )); do
    copyZip ${fileBase}/${oracleNameHead}_${c}of${oracleDiskCount}.zip
    
  done
}

checkDBExist()
{
    su - $oracleOSUserName -c ". ~/.bashrc;lsnrctl status" > oracleLsnrOutput
    grep "Instance \"$oracleInstanceName\", status READY" oracleLsnrOutput 
    if [ $? -eq 0 ];then
       echo "   Oracle is currently installed on the target machine and Instance [$oracleInstanceName] is created"
       return 0
    else
       echo "   Oracle is not currently installed on the target machine, proceed with install process"
       return 1
    fi
}


###############################
# Main
###############################

# set up local environment for this script
scriptName=$0
workingDir=`pwd -P`
envProp=/etc/env.properties
. $envProp

# get credentials
userCreds=$AUTO_PREP/UserParameters/userCreds.properties
. $userCreds

# get media properties
mediaProp=$AUTO_PREP/MediaParameters/media.properties
. $mediaProp

# get media version properties
. $AUTO_PREP/MediaParameters/mediaVersion.properties

if checkDBExist; then 
   echo "Database is installed on the system, exit installation step"
   exit 0
fi
if [ ! -d $oracleRootDirUnix ]; then
   mkdir -p $oracleRootDirUnix
   if [ $? -gt 0 ];then
      echo "ERROR: Failed to create oracleRootDirUnix [$$oracleRootDirUnix]"
      exit 1
   fi
   chmod 777 $oracleRootDirUnix
fi

if [ "ZIP" == $oracleMediaType ] || [ "FILE" == $oracleMediaType ]; then 
  echo "using [${oracleMediaType}] media location type"
else
  errMsg="["${oracleMediaType}"] media location type not supported"
  exitWithErrMsg "$errMsg"
fi


if [ "X"$nfsEnabled"X" == "XX" ]; then
  echo "no NFS drive exists, assuming any ZIP files are local"
  nfsEnabled=FALSE
else
  # need to check value
  if [ "FALSE" == $nfsEnabled ]; then
    echo "no NFS drive exists, assuming any ZIP files are local"
    nfsEnabled=FALSE
  elif [ "TRUE" == $nfsEnabled ]; then 
    echo "NFS drive exists, assuming any ZIP files are remote"
  else
    errMsg="["${nfsEnabled}"] nfsEnabled not supported"
    exitWithErrMsg "$errMsg"
  fi
fi

if [ "X"$createMediaOnly"X" == "XX" ]; then
  echo "installing Oracle"
  createMediaOnly=FALSE
else
  # need to check value
  if [ "FALSE" == $createMediaOnly ]; then
    echo "installing Oracle"
    createMediaOnly=FALSE
  elif [ "TRUE" == $createMediaOnly ]; then 
    echo "adding Oracle installer to media server"
  else
    errMsg="["${createMediaOnly}"] createMediaOnlyFlag not supported"
    exitWithErrMsg "$errMsg"
  fi
fi


if [ "X"$debugDoNotInstall"X" == "XX" ]; then
  echo "installing Oracle"
  debugDoNotInstall=FALSE
else
  # need to check value
  if [ "FALSE" == $debugDoNotInstall ]; then
    echo "installing oracle"
    debugDoNotInstall=FALSE
  elif [ "TRUE" == $debugDoNotInstall ]; then
    echo "DEBUG: will exit prior to installing oracle"
  else
    errMsg="["${debugDoNotInstall}"] debugDoNotInstallFlag not supported"
    exitWithErrMsg "$errMsg"
  fi
fi


## ======================================================================
## set $oracleInstallFile location
unzipFolderNeedsDeleted=FALSE
zipNeedsDeleted=FALSE
# "FILE" type is never going to used for middleware media
#  we have asked customer to use "Zip" type file format.
# We can delete below section code when we convert to python
if [ "FILE" == $oracleMediaType ]; then 
  ## using file ${mediaRoot}/${oracleMediaRoot}/${oracleVersion}/${oracleFile}
  if [ "XX" == "X"${mediaRoot}"X" ] || [ "XX" == "X"${oracleMediaRoot}"X" ] || [ "XX" == "X"${oracleVersion}"X" ] || [ "XX" == "X"${oracleNameHead}"X" ] ; then
    errMsg="$!mediaRoot [$mediaRoot] or $!oracleMediaRoot [$oracleMediaRoot] or $!oracleVersion [$oracleVersion] or $!oracleNameHead [$oracleNameHead] not set!"
    exitWithErrMsg "$errMsg"
  else
    oracleInstallFile="database/runInstaller"
  	oracleInstallFileLocation=${mediaRoot}"/"${oracleMediaRoot}"/"${oracleVersion}"/"${oracleInstallFile}
    echo "using file $!oracleInstallFileLocation [$oracleInstallFileLocation]"
  fi
fi
if [ "ZIP" == $oracleMediaType ]; then
  # need to calculate $oracleRepoZipFileLocation
  if [ "FILE" == $mediaAccess ]; then
    ## using file ${mediaRoot}/${oracleMediaRoot}/${oracleVersion}/${oracleFile}
    if [ "XX" == "X"${mediaRoot}"X" ] || [ "XX" == "X"${oracleMediaRoot}"X" ] || [ "XX" == "X"${oracleVersion}"X" ] || [ "XX" == "X"${oracleNameHead}"X" ] ; then
      errMsg="$!mediaRoot [$mediaRoot] or $!oracleMediaRoot [$oracleMediaRoot] or $!oracleVersion [$oracleVersion] or $!oracleNameHead [$oracleNameHead] not set!"
      exitWithErrMsg "$errMsg"
    else
      oracleRepoZipFileLocation=${mediaRoot}"/"${oracleMediaRoot}"/"${oracleVersion}
    fi
  fi
  if [ "HTTP" == $mediaAccess ] || [ "HTTPS" == $mediaAccess ]; then
    if [ "X"$oracleRepoFolder"X" == "XX" ]; then
      tempStr="notURL"
    else
      tempStr=`echo ${oracleRepoFolder} | cut -c -4`
    fi
    if [ "http" == $tempStr ]; then
      ## $oracleRepoFolder contains the full URL
      oracleRepoZipFolder=${oracleRepoFolder}"/"${oracleVersion}
    else
      # calculate the URL = ${mediaServer}/${oracleMediaRoot}/${oracleVersion}/${oracleFile}
      if [ "XX" == "X"${mediaServer}"X" ] || [ "XX" == "X"${oracleMediaRoot}"X" ] || [ "XX" == "X"${oracleVersion}"X" ] || [ "XX" == "X"${oracleNameHead}"X" ] ; then
        errMsg="$!mediaServer [$mediaServer] or $!oracleMediaRoot [$oracleMediaRoot] or $!oracleVersion [$oracleVersion] or $!oracleNameHead [$oracleNameHead] not set! "
        exitWithErrMsg "$errMsg"
      else
        oracleRepoZipFileLocation=${mediaServer}"/"${oracleMediaRoot}"/"${oracleVersion}
      fi
    fi
  fi  

  ## using zipped up file
  if [ "X"$oracleRepoZipFileLocation"X" == "XX" ]; then
    errMsg="$!oracleRepoZipFileLocation [$oracleRepoZipFileLocation] not set!"
    exitWithErrMsg "$errMsg"
  else
    echo "using zipped up file, $!oracleRepoZipFileLocation [$oracleRepoZipFileLocation]"    
    
    ## download the zip (http) OR copy the zip (nfs)
    downloadDir=${QD_InternalMedia}"/"${oracleMediaRoot}
    unzippedDir=${downloadDir}"/unzipped"
    oracleRepoDir=$downloadDir
    mkdir -p ${downloadDir}
    mkdir -p ${unzippedDir}
    cd ${downloadDir}
    tempStr=`echo ${oracleRepoZipFileLocation} | cut -c -4`
    timestamp=`date +%s`
    echo "DEBUG (ready to get ZIP) : current epoch time is ["$timestamp"] Sec"
    if [ "http" == $tempStr ]; then
      ## download the zip from the url
      getOracleZipFiles $oracleRepoZipFileLocation $oracleNameHead $oracleDiskCount
      zipNeedsDeleted=TRUE
    else
      # only download a copy if NFS has been setup
      if [ "TRUE" == ${nfsEnabled} ]; then
        # copy the zip
        copyOracleZipFiles $oracleRepoZipFileLocation $oracleNameHead $oracleDiskCount
        zipNeedsDeleted=TRUE
      fi
    fi
    timestamp=`date +%s`
    echo "DEBUG (ready to unzip ZIP) : current epoch time is ["$timestamp"] Sec"
    ## unzip
    if [ "http" == $tempStr ]; then
      echo "unzip the image zips"
      cd $downloadDir
      unzipOracleZipFiles $oracleNameHead $oracleDiskCount $unzippedDir
      unzipFolderNeedsDeleted=TRUE
    else
      if [ "TRUE" == ${nfsEnabled} ]; then
        cd $downloadDir
        unzipOracleZipFiles $oracleNameHead $oracleDiskCount $unzippedDir
        unzipFolderNeedsDeleted=TRUE
      else
        oracleRepoDir=$oracleRepoZipFileLocation
        unzipOracleZipFiles $oracleNameHead $oracleDiskCount $unzippedDir
        unzipFolderNeedsDeleted=TRUE
      fi
    fi
    timestamp=`date +%s`
    echo "DEBUG (TAR exploded) : current epoch time is ["$timestamp"] Sec"
  
    oracleInstallFolder=${unzippedDir}"/database/"
    oracleInstallFile="runInstaller"
    oracleInstallFileLocation=${oracleInstallFolder}"/"${oracleInstallFile}
    echo "using unzipped file, $!oracleInstallFileLocation [$oracleInstallFileLocation]"
    checkFileExists ${oracleInstallFileLocation}
  
  fi
fi


## =============================================================
## === exit if only creating a media server

if [ "TRUE" == $createMediaOnly ]; then
  echo "media server update done"
  exit 0
fi


## =============================================================
## === DEBUG early exit

if [ "TRUE" == $debugDoNotInstall ]; then
  echo "DEBUG: Exiting prior to installing oracle"
  exit 0
fi

if [ -d $oracleRootDirUnix ] 
then
	
	export ORACLE_TARGET_SERVER_HOST=$(hostname --long)

	temporacleALLPassword=`echo ${oracleALLPassword} | base64 -d`
	temporacleSYSPassword=`echo ${oracleSYSPassword} | base64 -d`
	temporacleDBSNMPPassword=`echo ${oracleDBSNMPPassword} | base64 -d`
	temporacleSYSTEMPassword=`echo ${oracleSYSTEMPassword} | base64 -d`
	temporaclePDBADMINPassword=`echo ${oraclePDBADMINPassword} | base64 -d`

	sed -i s%_oracleVersion_%$oracleVersion% ${workingDir}/Oracle12cServer.rsp	
	sed -i s%_oracleRootDir_%$oracleRootDirUnix% ${workingDir}/Oracle12cServer.rsp	
	sed -i s%ALL_PASSWORD%$temporacleALLPassword% ${workingDir}/Oracle12cServer.rsp
	sed -i s%SYS_PASSWORD%$temporacleSYSPassword% ${workingDir}/Oracle12cServer.rsp
	sed -i s%DBSNMP_PASSWORD%$temporacleDBSNMPPassword% ${workingDir}/Oracle12cServer.rsp
	sed -i s%SYSTEM_PASSWORD%$temporacleSYSTEMPassword% ${workingDir}/Oracle12cServer.rsp
	sed -i s%PDBADMIN_PASSWORD%$temporaclePDBADMINPassword% ${workingDir}/Oracle12cServer.rsp
	
#	HostDomainName=`hostname -d`
#	if [ "x$HostDomainNamex" == "xx" ];then
#		sed -i s%_GlobalDBName_%$oracleInstanceName% ${workingDir}/Oracle12cServer.rsp
#	else
#		sed -i s%_GlobalDBName_%${oracleInstanceName}.${HostDomainName}% ${workingDir}/Oracle12cServer.rsp
#	fi
	sed -i s%_InstanceName_%$oracleInstanceName% ${workingDir}/Oracle12cServer.rsp

	sed -i s%ALL_PASSWORD%$temporacleALLPassword% ${workingDir}/cfgrsp.properties
    sed -i s%SYS_PASSWORD%$temporacleSYSPassword% ${workingDir}/cfgrsp.properties
    sed -i s%DBSNMP_PASSWORD%$temporacleDBSNMPPassword% ${workingDir}/cfgrsp.properties
    sed -i s%SYSTEM_PASSWORD%$temporacleSYSTEMPassword% ${workingDir}/cfgrsp.properties
    sed -i s%PDBADMIN_PASSWORD%$temporaclePDBADMINPassword% ${workingDir}/cfgrsp.properties

    #Prepare oracle user
	tempPassword=`echo ${oracleOSUserPassword} | base64 -d`

	echo "oracleOSUserName: [$oracleOSUserName]"
	echo "oracleOSUserPassword: [**XX**]"
	echo "oracleOSUserPrimaryGroups: [$oracleOSUserPrimaryGroup] [$oracleOSUserAdditionalGroup]"

	# Creating group if the group does not exist on the system
	getent group $oracleOSUserPrimaryGroup
	if [ $? -eq 0 ]; then
  	 	echo "group [$oracleOSUserPrimaryGroup] exists"
	else
    	echo "creating group: [groupadd $oracleOSUserPrimaryGroup]"
     	groupadd $oracleOSUserPrimaryGroup
     	if [ $? -gt 0 ];
  		then
  		   echo "ERROR: Failed to create group [$oracleOSUserPrimaryGroup]"
  		   exit 1
  		fi
	fi

	# Creating group if the group does not exist on the system
	getent group $oracleOSUserAdditionalGroup
	if [ $? -eq 0 ]; then
   		echo "group [$oracleOSUserAdditionalGroup] exists"
	else
     	echo "creating group: [groupadd $oracleOSUserAdditionalGroup]"
     	groupadd $oracleOSUserAdditionalGroup
     	if [ $? -gt 0 ];
  		then
  		   echo "ERROR: Failed to create group [$oracleOSUserAdditionalGroup]"
  		   exit 1
  		fi
	fi

	#Create oracle OS user
	id $oracleOSUserName
	exitStatus=$?
	if [ $exitStatus -eq 0 ];then
		# check if the user belongs to a group already
  		echo user exist, checking if the user belongs to group [$oracleOSUserAdditionalGroup]
  		id $oracleOSUserName | grep $oracleOSUserAdditionalGroup
  		if [ $? -gt 0 ];
  		then
     		echo "Add user to group [$oracleOSUserAdditionalGroup]"
     		usermod -a -G $oracleOSUserAdditionalGroup $oracleOSUserName
     		if [ $? -gt 0 ];
     		then
         		echo "ERROR: Failed to add [$oracleOSUserName] to group [$oracleOSUserAdditionalGroup]"
         		exit 1
     		fi
     	else
     	    echo "User already belongs to group [$oracleOSUserAdditionalGroup]"
  		fi
  		echo "checking if the user belongs to group [$oracleOSUserPrimaryGroup]"
   		id $oracleOSUserName | grep $oracleOSUserPrimaryGroup
  		if [ $? -gt 0 ];
  		then
     		echo "Add user to group [$oracleOSUserPrimaryGroup]"
     		usermod -a -g $oracleOSUserPrimaryGroup $oracleOSUserName
     		if [ $? -gt 0 ];
     		then
         		echo "ERROR: Failed to add [$oracleOSUserName] to group [$oracleOSUserPrimaryGroup]"
         		exit 1
     		fi
  		fi
	else
  		useradd -d /home/$oracleOSUserName -m $oracleOSUserName -g $oracleOSUserPrimaryGroup -G $oracleOSUserAdditionalGroup
  		if [ $? -gt 0 ];
  		then
  		   echo "ERROR: Failed to create user or add user to a group"
  		   exit 1
  		fi
  		echo $tempPassword | passwd $oracleOSUserName --stdin
  		if [ $? -gt 0 ];
  		then
  		   echo "ERROR: Failed to set password for [$oracleOSUserName]"
  		   exit 1
  		fi
	fi

	
	echo " "
	echo "Creating swap space"
	#Create swap space, oracle groups, oracle user
	RAMSize=`grep MemTotal /proc/meminfo |awk '{print $2}'`
	if [ $RAMSize -lt 16000000 ]; then
		swapsize=$RAMSize
	else
		swapsize=15957904
    fi
	umask 022
	# use count value of "8174592" for 8GB RAM and "15957904" for 16GB RAM
	dd if=/dev/zero of=/swapfile1 bs=1024 count=$swapsize
	mkswap /swapfile1
	swapon /swapfile1
	if [ $? -gt 0 ];
  	then
  		 echo "ERROR: Failed to set swapfile1 "
  		 exit 1
  	fi
	echo " "
	echo "Show swap space"
	#Show swap space creation
	free -m;
	
	echo " "

	chmod 755 * ${workingDir}
	ls -la
	chmod 755 * /home
	ls -la
	
	echo " "
	echo "Replace sysctl.conf"
	#Replace sysctl.conf
    cp /etc/sysctl.conf /etc/sysctl_orig.conf
	cp -f ${workingDir}/sysctl.conf /etc/
	if [ $? -gt 0 ];
  	then
  		echo "ERROR: Failed to move [${workingDir}]/sysctl.conf /etc"
  		exit 1
  	fi
	
	/sbin/sysctl -p
	if [ $? -gt 0 ];
  	then
  		echo "ERROR: Failed to run [/sbin/sysctl -p]"
  		exit 1
  	fi
  	
	echo " "
	echo "Replace limits.conf"
	#Replace limits.conf
    cp /etc/security/limits.conf /etc/security/limits_orig.conf
    cp ${workingDir}/limits.conf ${workingDir}/temp_limits.conf
    sed -i s%_oracleOSUserName_%$oracleOSUserName% ${workingDir}/temp_limits.conf
	cp -f ${workingDir}/temp_limits.conf /etc/security/limits.conf
	if [ $? -gt 0 ];
  	then
  		echo "ERROR: Failed to move [${workingDir}]/limits.conf /etc/security/"
  		exit 1
  	fi
	echo " "
	echo "Run Oracle Install in silent mode..."
	#Install Oracle 12c Server using response file
	workingDir=$workingDir oracleInstallFileLocation=$oracleInstallFileLocation su -c '. /etc/env.properties;$oracleInstallFileLocation -silent -showProgress -waitforcompletion -nowelcome -responseFile ${workingDir}/Oracle12cServer.rsp' -- $oracleOSUserName
	if [ $? -gt 0 ];
  	then
  		echo "ERROR: Failed to install oracle"
  		exit 1
  	fi
		
	echo " "
	echo "Run Root scripts to clean up install..."
	#Run Root scripts to clean up install
	sh $oracleRootDirUnix/app/oraInventory/orainstRoot.sh
	if [ $? -gt 0 ];
  	then
  		echo "ERROR: Failed to run $oracleRootDirUnix/app/oraInventory/orainstRoot.sh"
  		exit 1
  	fi
	sh $oracleRootDirUnix/app/oracle/product/${oracleVersion}/dbhome_1/root.sh
	if [ $? -gt 0 ];
  	then
  		echo "ERROR: Failed to run $oracleRootDirUnix/app/oracle/product/${oracleVersion}/dbhome_1/root.sh"
  		exit 1
  	fi	
	echo " "
	echo "Set ORACLE_HOME and append ORACLE_HOME/bin to PATH for root"
	echo "# User specific environment and startup programs" >> ~/.bashrc
	echo "TMP=/tmp;" >> ~/.bashrc
	echo "export TMP;" >> ~/.bashrc
	echo "TMPDIR=$TMP;" >> ~/.bashrc
	echo "export TMPDIR;" >> ~/.bashrc
	echo "ORACLE_HOME=$oracleRootDirUnix/app/oracle/product/${oracleVersion}/dbhome_1" >> ~/.bashrc
	echo "export ORACLE_HOME" >> ~/.bashrc
	echo 'PATH=$PATH:$HOME/bin:$ORACLE_HOME/bin' >> ~/.bashrc
	echo "export PATH" >> ~/.bashrc
	echo "unset USERNAME" >> ~/.bashrc
	
	echo " "
	echo "Run .bashrc file for root"
	. ~/.bashrc
	if [ $? -gt 0 ];
  	then
  		echo "ERROR: Failed to source root .bashrc file"
  		exit 1
  	fi		
	echo "ORACLE_HOME ="
	echo $ORACLE_HOME
	echo "PATH ="
	echo $PATH
	 
	echo " "
	echo "Set .bash_profile and .bashrc for the user oracle"
	sed -i s%_oracleOSUser_%$oracleOSUserName% ${workingDir}/bash_profile 
	cp -f ${workingDir}/bash_profile /home/${oracleOSUserName}/.bash_profile
	if [ $? -gt 0 ];
  	then
  		echo "ERROR: Failed to cp -f ${workingDir}/bash_profile /home/${oracleOSUserName}/.bash_profile"
  		exit 1
  	fi	
  	
  	sed -i s%_oracleRootDirUnix_%$oracleRootDirUnix% ${workingDir}/bashrc
  	sed -i s%_DBVersion_%$oracleVersion% ${workingDir}/bashrc
  	sed -i s%_oracleOSUser_%$oracleOSUserName% ${workingDir}/bashrc  		
	cp -f ${workingDir}/bashrc /home/${oracleOSUserName}/.bashrc
	if [ $? -gt 0 ];
  	then
  		echo "ERROR: Failed to cp -f ${workingDir}/bashrc /home/${oracleOSUserName}/.bashrc"
  		exit 1
  	fi			
	echo "configure listener and create databases based on the installation response file.  It creates the DB instance directory"
	oracleVersion=$oracleVersion workingDir=$workingDir oracleRootDirUnix=$oracleRootDirUnix su -c '. /etc/env.properties;$oracleRootDirUnix/app/oracle/product/$oracleVersion/dbhome_1/cfgtoollogs/configToolAllCommands RESPONSE_FILE=${workingDir}/cfgrsp.properties' -- $oracleOSUserName
	if [ $? -gt 0 ];
  	then
  		echo "ERROR: Failed to configure listener"
  		exit 1
  	fi
  		
	echo " "
	echo "Start Oracle Server"
	oracleVersion=$oracleVersion oracleRootDirUnix=$oracleRootDirUnix su -c '$oracleRootDirUnix/app/oracle/product/$oracleVersion/dbhome_1/bin/dbstart $ORACLE_HOME' -- $oracleOSUserName
	if [ $? -gt 0 ];
  	then
  		echo "ERROR: Failed to start oracle server"
  		exit 1
  	fi	
	echo " "
	echo "Adding Oracle Service scripts"
	chmod 755 ${workingDir}/ora_start.sh
	chmod 755 ${workingDir}/ora_stop.sh
	
	sed -i s%_OracleInstallDir_%$oracleRootDirUnix% ${workingDir}/ora_start.sh
	sed -i s%_OracleInstallDir_%$oracleRootDirUnix% ${workingDir}/ora_stop.sh
	sed -i s%_OracleOSUser_%${oracleOSUserName}% ${workingDir}/oracle
	oracleOSUserName=$oracleOSUserName su -c 'mkdir /home/${oracleOSUserName}/scripts' -- ${oracleOSUserName}
	cp -f ${workingDir}/ora_start.sh /home/${oracleOSUserName}/scripts/
	cp -f ${workingDir}/ora_stop.sh /home/${oracleOSUserName}/scripts/
	
	####################################
	# may need to check OS 
	bRH6flag=FALSE
	bRH7flag=FALSE

	RHversion=`cat /etc/redhat-release`
	echo "Current Red Hat Release $RHversion"

	echo $RHversion | grep "Maipo"
	if [ $? -eq 0 ]; then
  		echo "OS is Red Hat 7"
  	bRH7flag=TRUE
	fi
	echo $RHversion | grep "Santiago"
	if [ $? -eq 0 ]; then
  		echo "OS is Red Hat 6"
  		bRH6flag=TRUE
	fi
	
	if [ $bRH6flag == TRUE ]; then	
		echo " "
		echo "Adding Oracle Service"
		cp -f ${workingDir}/oracle /etc/init.d/
		chmod 755 /etc/init.d/oracle
	`	chkconfig --add oracle`
	
		echo " "
		echo "Adding Oracle DB Startup Service"
		mv ${workingDir}/oracleDBstartUp /etc/init.d/
		chmod 755 /etc/init.d/oracleDBstartUp
	`	chkconfig --add oracleDBstartUp`
	fi
	
	if [ $bRH7flag == TRUE ]; then
	   	echo "*** Setup $workingDir/oracle systemd"
    	if [ ! -d /usr/lib/systemd/scripts/ ];then
      		mkdir /usr/lib/systemd/scripts/
    	fi
    	cp -f ${workingDir}/oracle /usr/lib/systemd/scripts/
    	chmod o+x /usr/lib/systemd/scripts/oracle
    	cp $workingDir/oracle.service /usr/lib/systemd/system/
    	systemctl enable oracle.service
    	echo "checking start up scripts"
    	ls -l /usr/lib/systemd/scripts/oracle
    	ls -la /etc/systemd/system/multi-user.target.wants | grep oracle.service
	fi
	
	echo "------Setting Oracle Databases to Restart on VM reboot---------"
    sed -i s%"$ORACLE_HOME:N"%"$ORACLE_HOME:Y"% /etc/oratab
	
	cd $ORACLE_HOME/install
	cat portlist.ini
		
	echo " "
	echo "Remove  ${QD_InternalMedia}/${oracleMediaRoot}"
	rm -rf ${QD_InternalMedia}/${oracleMediaRoot}
	
	if [ "$oraclePort" != "1521" ]; then
		#Register local listner
		chmod 755 $workingDir/registerLocalListner.sh
		temporacleSYSPassword=$temporacleSYSPassword workingDir=$workingDir su - $oracleOSUserName -c ". ~/.bashrc;. /etc/env.properties;$workingDir/registerLocalListner.sh $temporacleSYSPassword"
		if [ $? -gt 0 ];
  		then
  			echo "ERROR: Failed register local listner"
  			exit 1
  		fi	
  		
  		#stop listner
  		su - $oracleOSUserName -c ". ~/.bashrc;. /etc/env.properties;lsnrctl stop"
		if [ $? -gt 0 ];
  		then
  			echo "ERROR: Failed stop listner"
  			exit 1
  		fi	
  		#Modify port
  		echo "Modify oraclePort [$oraclePort] in listener and tnsnames file"
  		sed -i s%1521%$oraclePort% ${ORACLE_HOME}/network/admin/listener.ora
  		sed -i s%1521%$oraclePort% ${ORACLE_HOME}/network/admin/tnsnames.ora
  		su - $oracleOSUserName -c ". ~/.bashrc;. /etc/env.properties;lsnrctl start"
  		
  		#restart db
  		ORACLE_HOME=$ORACLE_HOME su - $oracleOSUserName -c ". ~/.bashrc;. /etc/env.properties;$ORACLE_HOME/bin/dbshut $ORACLE_HOME"
		if [ $? -gt 0 ];
  		then
  			echo "ERROR: Failed shutdown database"
  			exit 1
  		fi	
  		
  		ORACLE_HOME=$ORACLE_HOME su - $oracleOSUserName -c ". ~/.bashrc;. /etc/env.properties;$ORACLE_HOME/bin/dbstart $ORACLE_HOME"
		if [ $? -gt 0 ];
  		then
  			echo "ERROR: Failed start database"
  			exit 1
  		fi	
  		
  		su - $oracleOSUserName -c ". ~/.bashrc;. /etc/env.properties;lsnrctl status"
  	fi
  					
else
	echo " "
	echo "Stopping ... can not proceed with out [$oracleRootDirUnix] directory."
	exit 1

fi
exit 0

