#!/bin/bash
#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2016. All Rights Reserved.
#  
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************

#############
## INPUTS : (2 key=value pairs)
#
# UCD_SERVER=<https://ucdServer:port>
# UCD_TOKEN=<ucdUserToken>
# 
# also expect JAVA_HOME to be set and exported prior to execution-
#############

#############
## FUNCTIONS 
#
# - $errMsg is used as a return code in some functions
#############

# all cli / rest api calls come through this function
# eval $tempCommand used to ensure the command is correctly formatted
# $errCode : return code (integer) is available to step after calling step
#            and should be used to stop execution on failure
runCommand()
{
  tempCommand=$1
  printNotExecute=0
  errCode="blank"
  if [ 1 -eq $printNotExecute ]; then
    echo COMMAND: [${tempCommand}]
	errCode=$?
  else
    eval $tempCommand
	errCode=$?
  fi
  echo "returnCode ["$errCode"]"
}

setDefaultInputs()
{
  UCD_SERVER=defaultUCD_SERVER
  UCD_TOKEN=defaultUCD_TOKEN
}

checkForNoDefaultInputs()
{
  if [ "$UCD_SERVER" = "defaultUCD_SERVER" ]; then
    errMsg="UCD_SERVER [$UCD_SERVER] still contains its default value"
    exitWithErrMsg "$errMsg"
  fi
  if [ "$UCD_TOKEN" = "defaultUCD_TOKEN" ]; then
    errMsg="UCD_TOKEN [$UCD_TOKEN] still contains its default value"
    exitWithErrMsg "$errMsg"
  fi

  echo "all inputs are set"
}

checkJavaHome()
{
  # -- ucd cli client needs JAVA_HOME set, so check it exists
  echo "JAVA_HOME currently [${JAVA_HOME}]"
  if [ "" = "${JAVA_HOME}" ]; then
    errMsg="You must have JAVA_HOME set (exported) on your environment to use the uDeploy client."
    exitWithErrMsg "$errMsg"
  else
    echo "check [${JAVA_HOME}] folder exists"
    checkFolderExists ${JAVA_HOME}
  fi
}

## usage exitWithOkMsg tempString
exitWithOkMsg()
{
  tempString=$1
  echo "-- message: ${tempString}"
  echo "+++ SUCCESS : EOF [${scriptName}] +++"
  exit 0
}

## usage exitWithErrMsg tempString
exitWithErrMsg()
{
  tempString=$1
  echo "-- ERROR [${scriptName}][${currentStage}] --"
  echo "-- message: $tempString"
  exit 1
}

checkFileExists()
{
  tempFile=$1
  if [ -f $tempFile ]; then
    echo "  file [$tempFile] exists"
  else
    errMsg="file [$tempFile] does not exist"
    exitWithErrMsg "$errMsg"
  fi
}

checkFolderExists()
{
  tempFolder=$1
  if [ -d $tempFolder ]; then
    echo "  folder [$tempFolder] exists"
  else
    errMsg="folder [$tempFolder] does not exist"
    exitWithErrMsg "$errMsg"
  fi
}

## usage deleteFile (force) from folder
deleteFile()
{
  tempFolder=$1
  tempFile=$2
  fileToDelete=${tempFolder}"/"${tempFile}

  if [ -f ${fileToDelete} ]; then
    echo "delete file ["${fileToDelete}"]"
    rm -f ${fileToDelete}
    if [ -f ${fileToDelete} ]; then
      errMsg="failed to delete file"
      exitWithErrMsg "${errMsg}"
    else
      echo "successfully deleted [${fileToDelete}]"
    fi
  else
    echo "file [${fileToDelete}] does not exist, so cannot be deleted"
  fi
}

## usage unzip contents to target folder
## using the [-o] option to force overwriting of any existing files
unzipZip()
{
  zipFile=$1
  targetDir=$2

  COMMAND="unzip -o "${zipFile}" -d "${targetDir}
  echo "executing command [${COMMAND}]"
  runCommand "$COMMAND"
  # errCode=$?
  if [ $errCode -eq 0 ]; then
    echo "unzip successful" 
  else
    errMsg="[${errCode}] failed to unzip  ["${zipFile}"]"
    exitWithErrMsg "$errMsg"
  fi
}

## usage wgetFile from URL to specific target destination
wgetFile()
{
  tempWgetSource=$1
  tempWgetTarget=$2
  
  echo "download [${tempWgetSource}] to [${tempWgetTarget}]"
  COMMAND="wget -nv --no-check-certificate "${tempWgetSource}" -O "${tempWgetTarget}
  echo "executing command [${COMMAND}]"
  runCommand "$COMMAND"
  # errCode=$?
  if [ $errCode -eq 0 ]; then
    echo "download successful"
	checkFileExists ${tempWgetTarget}
  else
    errMsg="[${errCode}] Failed to download [${tempWgetSource}]"
    exitWithErrMsg "${errMsg}"
  fi
}

# TRUE | FALSE is there just one QD json in the folder
# file name is Rational*.json
justOneRationalJsonFile()
{
  # folder=$1
  
  echo "ensure there is only one Rational QD JSON file"
  tempFileName=`ls $1 | grep -e '^Rational' | grep -e '\.json'`
  tempFileCount=`ls $1 | grep -e '^Rational' | grep -e '\.json' | wc -w`

  if [ $tempFileCount -eq 1 ]; then
    echo "folder [$1] contains a single file [$tempFileName]"
  else
	echo "folder [$1] contains [$tempFileCount] files"
    echo $tempFileName
    errMsg="expected just a single file, found [$tempFileCount]"
    exitWithErrMsg "$errMsg"
  fi
}


# TRUE | FALSE is there at least one QD component zip in the folder
# file name is Rational_QD_*.zip
atLeastOneRationalZipFile()
{
  # folder=$1
  
  echo "ensure there is at least one Rational QD ZIP file"
  tempFileNames=`ls $1 | grep -e '^Rational_QD_' | grep -e '\.zip'`
  tempFileCount=`ls $1 | grep -e '^Rational_QD_' | grep -e '\.zip' | wc -w`

  if [ $tempFileCount -eq 0 ]; then
    errMsg="expected at least one file in folder [$1], found [$tempFileCount]"
    exitWithErrMsg "$errMsg"
  else
    echo "folder [$1] contains [$tempFileCount] files"
    echo $tempFileNames
  fi
}



# --- functions using UCD cli ---

# uses globals $UCD_TOKEN $UCD_SERVER $UCDCLIEXE
createResource()
{
  tempResourceFile=$1
  
  echo "  creating temporary resource defined in ["${tempResourceFile}"]"
  COMMAND=${UCDCLIEXE}" -authtoken "${UCD_TOKEN}" -weburl "${UCD_SERVER}" createResource "${tempResourceFile}
  echo "executing command [${COMMAND}]"
  # -- success and ucd returns a json representation of the resource
  # -- failure and no json is returned
  runCommand "$COMMAND"
  # errCode=$?
  if [ $errCode -eq 0 ]; then
    echo "  successfully created resource" 
  else
    errMsg="[$errCode] failed to create resource"
    exitWithErrMsg "$errMsg"
  fi
}


# uses globals $UCD_TOKEN $UCD_SERVER $UCDCLIEXE
deleteResource()
{
  tempResource=$1
  
  echo "  deleting resource  ["${tempResource}"]"
  COMMAND=${UCDCLIEXE}" -authtoken "${UCD_TOKEN}" -weburl "${UCD_SERVER}" deleteResource -resource "${tempResource}
  echo "executing command [${COMMAND}]"
  runCommand "$COMMAND"
  # errCode=$?
  if [ $errCode -eq 0 ]; then
    echo "  successfully deleteded resource"
  else
    errMsg="[$errCode] failed to delete resource"
    exitWithErrMsg "$errMsg"
  fi
}

# uses globals $UCD_TOKEN $UCD_SERVER $UCDCLIEXE
addTagToResource()
{

  resourceName=$1
  tagName=$2
  tagColour=$3
  
  echo "  adding tag ["${tagName}"] with colour ["${tagColour}"] to resource ["${resourceName}"]"
  COMMAND=${UCDCLIEXE}" -authtoken "${UCD_TOKEN}" -weburl "${UCD_SERVER}" addTagToResource -resource "${resourceName}" -tag "${tagName}" -color "${tagColour}
  echo "executing command [${COMMAND}]"
  runCommand "$COMMAND"
  # errCode=$?
  if [ $errCode -eq 0 ]; then
    echo "  successfully added tag"
  else
    errMsg="[$errCode] failed to add tag"
    exitWithErrMsg "$errMsg"
  fi
}

# uses globals $UCD_TOKEN $UCD_SERVER $UCDCLIEXE
createUser()
{
  tempQDUser=$1
  tempQDUserFile=$2
 
  echo "  adding user ["${tempQDUser}"] to UCD"
  COMMAND=${UCDCLIEXE}" -authtoken "${UCD_TOKEN}" -weburl "${UCD_SERVER}" createUser "${tempQDUserFile}
  echo "executing command [${COMMAND}]"
  runCommand "$COMMAND"
  # errCode=$?
  if [ $errCode -eq 0 ]; then
    echo "  successfully created user"
  else
    errMsg="[$errCode] failed to create user"
    exitWithErrMsg "$errMsg"
  fi
}

# uses globals $UCD_TOKEN $UCD_SERVER $UCDCLIEXE
# note that single quotes are used in $COMMAND so that it can contain double quotes
createTeam()
{
  tempQDTeam=$1
 
  echo "  adding team ["${tempQDTeam}"] to UCD"
  COMMAND=${UCDCLIEXE}' -authtoken '${UCD_TOKEN}' -weburl '${UCD_SERVER}' createTeam -team "'${tempQDTeam}'" -description "Created automagically"'
  echo "executing command [${COMMAND}]"
  runCommand "$COMMAND"
  # errCode=$?
  if [ $errCode -eq 0 ]; then
    echo "  successfully created team"
  else
    errMsg="[$errCode] failed to create team"
    exitWithErrMsg "$errMsg"
  fi
}

# uses globals $UCD_TOKEN $UCD_SERVER $UCDCLIEXE
# note that single quotes are used in $COMMAND so that it can contain double quotes
addUserToTeam()
{
  tempQDuser=$1
  tempUserRole=$2  
  tempQDTeam=$3
 
  echo "  adding user ["${tempQDuser}"] with role ["${tempUserRole}"] to team ["${tempQDTeam}"]"
  COMMAND=${UCDCLIEXE}' -authtoken '${UCD_TOKEN}' -weburl '${UCD_SERVER}' addUserToTeam -team "'${tempQDTeam}'" -user '${tempQDuser}' -role "'${tempUserRole}'"'
  echo "executing command [${COMMAND}]"
  runCommand "$COMMAND"
  # errCode=$?
  if [ $errCode -eq 0 ]; then
    echo "  successfully added user to team"
  else
    errMsg="[$errCode] failed to add user team"
    exitWithErrMsg "$errMsg"
  fi
}

# uses globals $UCD_TOKEN $UCD_SERVER $UCDCLIEXE
# note that single quotes are used in $COMMAND so that it can contain double quotes
addResourceToTeam()
{
  tempQDresource=$1
  tempQDTeam=$2

  echo "  adding resource ["${tempQDresource}"] to team ["${tempQDTeam}"]"
  COMMAND=${UCDCLIEXE}' -authtoken '${UCD_TOKEN}' -weburl '${UCD_SERVER}' addResourceToTeam -resource '${tempQDresource}' -team "'${tempQDTeam}'"'
  echo "executing command [${COMMAND}]"
  runCommand "$COMMAND"
  # errCode=$?
  if [ $errCode -eq 0 ]; then
    echo "  successfully added resource to team"
  else
    errMsg="[$errCode] failed to add resource team"
    exitWithErrMsg "$errMsg"
  fi
}

# uses globals $UCD_TOKEN $UCD_SERVER $UCDCLIEXE
addComponentVersion()
{
  componentName=$1
  componentVersionName=$2
  
  echo "  adding version ["${componentVersionName}"] to component ["${componentName}"]"
  COMMAND=${UCDCLIEXE}" -authtoken "${UCD_TOKEN}" -weburl "${UCD_SERVER}" createVersion -component "${componentName}" -name "${componentVersionName}
  echo "executing command [${COMMAND}]"
  # -- success and ucd returns a json representation of the resource
  # -- failure and no json is returned
  runCommand "$COMMAND"
  # errCode=$?
  if [ $errCode -eq 0 ]; then
    echo "  successfully created component version"
  else
    errMsg="[$errCode] failed to add version"
    exitWithErrMsg "$errMsg"
  fi
}


# uses globals $UCD_TOKEN $UCD_SERVER $UCDCLIEXE
addVersionFiles()
{
  componentName=$1
  componentVersionName=$2
  filesFolder=$3
  
  echo "  adding the files from ["${filesFolder}"] for version ["${componentVersionName}"] to component ["${componentName}"]"
  COMMAND=${UCDCLIEXE}" -authtoken "${UCD_TOKEN}" -weburl "${UCD_SERVER}" addVersionFiles -component "${componentName}" -version "${componentVersionName}" -base "${filesFolder}
  echo "executing command [${COMMAND}]"
  runCommand "$COMMAND"
  # errCode=$?
  if [ $errCode -eq 0 ]; then
    echo "  successful" 
  else
    errMsg="[$errCode] failed to add files"
    exitWithErrMsg "$errMsg"
  fi
}

# uses globals $UCD_TOKEN $UCD_SERVER $UCDCLIEXE
# note that single quotes are used in $COMMAND so that it can contain double quotes
createEnvironment()
{
  tempApplicationName=$1
  tempEnvironmentName=$2
  
  echo "  creating environment ["${tempEnvironmentName}"] in application ["${tempApplicationName}"]" 
  COMMAND=${UCDCLIEXE}' -authtoken '${UCD_TOKEN}' -weburl '${UCD_SERVER}' createEnvironment -application '${tempApplicationName}' -name "'${tempEnvironmentName}'"'
  echo "executing command [${COMMAND}]"
  runCommand "$COMMAND"
  # errCode=$?
  if [ $errCode -eq 0 ]; then
    echo "  successful" 
  else
    errMsg="[$errCode] failed to create environment"
    exitWithErrMsg "$errMsg"
  fi
}

# uses globals $UCD_TOKEN $UCD_SERVER $UCDCLIEXE
# note that single quotes are used in $COMMAND so that it can contain double quotes
addEnvironmentToTeam()
{
  tempApplicationName=$1
  tempEnvironmentName=$2
  tempQDTeam=$3
  
  echo "  adding environment ["${tempEnvironmentName}"] in application ["${tempApplicationName}"] to team ["${tempQDTeam}"]"
  COMMAND=${UCDCLIEXE}' -authtoken '${UCD_TOKEN}' -weburl '${UCD_SERVER}' addEnvironmentToTeam -application '${tempApplicationName}' -environment "'${tempEnvironmentName}'" -team "'${tempQDTeam}'"'
  echo "executing command [${COMMAND}]"
  runCommand "$COMMAND"
  # errCode=$?
  if [ $errCode -eq 0 ]; then
    echo "  successful" 
  else
    errMsg="[$errCode] failed to create environment"
    exitWithErrMsg "$errMsg"
  fi
}


# --- functions using UCD REST api ---

# uses REST API and globals $UCD_TOKEN $UCD_SERVER
# note that single quotes are used in $COMMAND so that it can contain double quotes
createSecurityRole()
{
  tempSecurityRole=$1
 
  echo "  adding security role ["${tempSecurityRole}"] to UCD"
  COMMAND='curl -v -k --compressed -H "Accept:*/*" -H "Accept-Encoding:gzip, deflate, br" -H "Accept-Language:en-US,en;q=0.8" -H "Connection:keep-alive" -H "Content-Type:application/json" -d {"name":"'${tempSecurityRole}'"} -u PasswordIsAuthToken:{"token":"'$UCD_TOKEN'"} '$UCD_SERVER'/security/role'
  echo "executing command [${COMMAND}]"
  runCommand "$COMMAND"
  # errCode=$?
  if [ $errCode -eq 0 ]; then
    echo "  successfully created security role"
  else
    errMsg="[$errCode] failed to create security role"
    exitWithErrMsg "$errMsg"
  fi
}

# uses REST API and globals $UCD_TOKEN $UCD_SERVER $UCDCLIEXE
# note that single quotes are used in $COMMAND so that it can contain double quotes
importApplication()
{
  tempApplicationName=$1
  tempApplicationFile=$2

  echo "  importing application from ["${tempApplicationFile}"] into UCD"
  COMMAND='curl -m 15 -v -k -u PasswordIsAuthToken:{"token":"'$UCD_TOKEN'"} -F "file=@'${tempApplicationFile}'" -F "filename='${tempApplicationFile}'" '$UCD_SERVER'/rest/deploy/application/import?upgradeType=CREATE_NEW_IF_EXISTS&compTempUpgradeType=CREATE_NEW_IF_EXISTS&processUpgradeType=CREATE_NEW_IF_EXISTS'
  echo "executing command [${COMMAND}]"
  runCommand "$COMMAND"
  # errCode=$?
  if [ $errCode -eq 0 ]; then
    echo "  waiting 1 minute before confirming application import"
  else
    errMsg="[$errCode] failed to import application"
    exitWithErrMsg "$errMsg"
  fi
  
  sleep 60
  
  echo "  confirming import of application ["${tempApplicationName}"]"
  COMMAND=${UCDCLIEXE}' -authtoken '${UCD_TOKEN}' -weburl '${UCD_SERVER}' getApplication -application "'${tempApplicationName}'"'
  echo "executing command [${COMMAND}]"
  runCommand "$COMMAND"
  # errCode=$?
  if [ $errCode -eq 0 ]; then
    echo "  application import successful" 
  else
    errMsg="[$errCode] something went wrong with application import"
    exitWithErrMsg "$errMsg"
  fi
}


# --- functions using java ---

# uses globals $UCD_TOKEN $UCD_SERVER 
createDefaultBlueprints()
{
  tempApplicationName=$1

  echo "  creating default blueprints in application ["${tempApplicationName}"]"
  COMMAND="java -cp  blueprint.jar:ucdapi.jar:working/udclient/udclient.jar com.persistent.nvb.qd.utils.blueprint.CreateDefaultBlueprints $UCD_SERVER $UCD_TOKEN ${tempApplicationName}"
  echo "executing command [${COMMAND}]"
  runCommand "$COMMAND"
  # errCode=$?
  if [ $errCode -eq 0 ]; then
    echo "  successful" 
  else
    errMsg="[$errCode] failed to create default blueprints"
    exitWithErrMsg "$errMsg"
  fi
}


#############
## CONSTANTS 
#############

currentDir=`pwd -P`
workingDir=${currentDir}"/working"
scriptName=$0

# - location is known as its getting downloaded from $UCD_SERVER
UCDCLIEXE=${workingDir}"/udclient/udclient"

# UCD tag colours
wasColour=9932CC
ihsColour=$wasColour
dbColour=000000
appColour=DDA0DD


####################
##                ##
## ==== MAIN ==== ##
##                ##
####################

# -- stage 1: get user inputs --
# ------------------------------
echo "==================================================================="
echo "====================== STAGE 1 : user inputs ======================"
echo "==================================================================="
currentStage="stage 1"
# -- set default values
echo "setting default inputs values"
setDefaultInputs
# -- process user inputs
echo "reading inputs values"
if [ $# -ne 2 ]; then
  errMsg="ERROR: script "${scriptName}" requires 2 arguments: UCD_SERVER=<https://ucdServer:port> UCD_TOKEN=<ucdUserToken>"
  exitWithErrMsg "$errMsg"
fi

while [ $# -gt 0 ]; do
   key=`echo $1 | cut -d'=' -f 1`
   value=`echo $1 | cut -d'=' -f 1 --complement`
   echo "[$key] [$value]"
   if [ "UCD_SERVER"  = "$key" ]; then
      UCD_SERVER=$value
   fi
   if [ "UCD_TOKEN"  = "$key" ]; then
      UCD_TOKEN=$value
   fi
   
   shift
done

# -- check all the defaults changed
echo "checking default inputs values are updated"
checkForNoDefaultInputs

# -- ucd cli client needs JAVA_HOME set, so check it exists
checkJavaHome


# -- stage 2: get UCD cli exe --
# ------------------------------
echo "==================================================================="
echo "================ STAGE 2 : download ucd cli client ================"
echo "==================================================================="
currentStage="stage 2"
# Create working directory
if [ ! -d ${workingDir} ]; then
    echo Creating working directory [${workingDir}]
    mkdir ${workingDir}
fi
checkFolderExists ${workingDir}

# Install UCD Client
# - download from UCD server
if [ ! -f ${workingDir}/udclient.zip ]; then
    echo Fetching UCD CLI zip
	wgetFile $UCD_SERVER/tools/udclient.zip ${workingDir}/udclient.zip
fi
# - unzip contents
if [ ! -d ${workingDir}/udclient ]; then
    echo Installing UCD CLI
    unzipZip ${workingDir}/udclient.zip ${workingDir}
fi
# - confirm "exe" exists
checkFileExists ${UCDCLIEXE}
 

# -- stage 3: Create "Rational QD CLM" team --
# --------------------------------------------
echo "==================================================================="
echo "================= STAGE 3 : create UCD QD CLM team ================"
echo "==================================================================="
currentStage="stage 3"
QDteam="Rational QD CLM"
QDmemberRole="Team Member"

echo Creating Rational QD CLM team
createTeam "${QDteam}"

# Add user admin to team as a UCD Administrator
echo Adding UCD [admin] user to [${QDteam}] team with [Administrator] role
addUserToTeam admin Administrator "${QDteam}"

# Create "Team Member" role
echo Creating Team Member role
createSecurityRole "${QDmemberRole}"

# Create ibm user and add to team
newUser="ibm"
echo Creating user [${newUser}] and adding it to team [${QDteam}] with role [${QDmemberRole}]
tempUserJsonFileName="user.json"
tempUserJsonFile=${workingDir}"/"${tempUserJsonFileName}
# delete temp file if it exists 
deleteFile ${workingDir} ${tempUserJsonFileName}
# create a new file containing user information
echo "create temp json file [${tempUserJsonFile}]"
echo '{"name":"'${newUser}'","password":"persistent","authenticationRealm":"Internal Security"}' > ${tempUserJsonFile}
sleep 1
echo "dumping contents of [${tempUserJsonFile}]"
cat ${tempUserJsonFile}
# create the new user and add to team
createUser ${newUser} ${tempUserJsonFile}
# delete temp file 
deleteFile ${workingDir} ${tempUserJsonFileName}
# add the new user to team
addUserToTeam ${newUser} "${QDmemberRole}" "${QDteam}"


# -- stage 4: Import QD application into UCD --
# ---------------------------------------------
echo "==================================================================="
echo "================= STAGE 4 : import QD application ================="
echo "==================================================================="
currentStage="stage 4"
echo
echo
# only progress if there is just one json file matching [Rational*.json]
justOneRationalJsonFile ${currentDir}
# import the json file into UCD
appFile=$(ls --file-type | grep -e 'Rational.*\.json')
appName=$(echo $appFile |  sed 's/v[^v]*$//')
echo Importing QD application [${appName}] from [${appFile}]
importApplication ${appName} ${appFile}


# -- stage 5: Import components into QD application --
# ----------------------------------------------------
echo "==================================================================="
echo "================== STAGE 5 : import QD components ================="
echo "==================================================================="
currentStage="stage 5"
# only progress if there is at least one zip file matching [Rational_QD_*.json]
atLeastOneRationalZipFile ${currentDir}
# add the components and version-ed scripts into UCD
for zipname in $( ls | grep ^Rational_QD_.*zip$ ); do
    component=$( echo $zipname | sed 's/\(^.*\)_\([^_]*\)_artifacts.zip$/\1/' )
    version=$( echo $zipname | sed 's/\(^.*\)_\([^_]*\)_artifacts.zip$/\2/' )
    echo Creating component [$component] version [$version] including scripts
	tempBaseDir=${workingDir}/$component/$version
    if [ ! -d ${tempBaseDir} ]; then
        mkdir -p ${tempBaseDir}
        unzipZip $zipname ${tempBaseDir}

        echo "  -- ${component} add version"
        addComponentVersion ${component} ${version}
        echo "  -- ${component} add files"
        addVersionFiles ${component} ${version} ${tempBaseDir}
    fi
done


# -- stage 6: Creating tags --
# ----------------------------
echo "==================================================================="
echo "==================== STAGE 6 : create QD tags ====================="
echo "==================================================================="
currentStage="stage 6"
echo Creating tags
# Create temporary resource
tempResourceName="rational-qd-tag-resource"
tempResourceJsonFileName="tempResource.json"
tempResourceJsonFile=${workingDir}"/"${tempResourceJsonFileName}
# delete temp file if it exists 
deleteFile ${workingDir} ${tempResourceJsonFileName}
# create a new file containing user information
echo "create temp json file [${tempResourceJsonFile}]"
echo '{ "name": "'${tempResourceName}'", "description": "temporary resource to create tags" }' > ${tempResourceJsonFile}
sleep 1
echo "dumping contents of [${tempResourceJsonFile}]"
cat ${tempResourceJsonFile}
# create the new temporary resource
echo Creating temporary resource
createResource ${tempResourceJsonFile}

# Add tags
tags="WAS IHS"
for tag in $tags ; do
    echo Creating tag $tag
	addTagToResource "/"${tempResourceName} $tag $wasColour
done

tags="DB"
for tag in $tags ; do
    echo Creating tag $tag
	addTagToResource "/"${tempResourceName} $tag $dbColour
done

tags="JTS CCM QM RM DCC LDX GC RELM RS LQE DM"
for tag in $tags ; do
    echo Creating tag $tag
	addTagToResource "/"${tempResourceName} $tag $appColour
done

# delete temporary resource from UCD
echo "delete the temporary resource from UCD"
deleteResource "/"${tempResourceName}
# delete temporary resource file
echo "delete the temporary resource file"
deleteFile ${workingDir} ${tempResourceJsonFileName}


# -- stage 7: Creating default environment --
# -------------------------------------------
# Create default resource group
echo "==================================================================="
echo "============= STAGE 7 : create QD default environment ============="
echo "==================================================================="
currentStage="stage 7"
echo Creating default resource group

# Create temporary resource
tempResourceName="default-resource-group"
tempResourceJsonFileName="tempResource.json"
tempResourceJsonFile=${workingDir}"/"${tempResourceJsonFileName}
# delete temp file if it exists 
deleteFile ${workingDir} ${tempResourceJsonFileName}
# create a new file containing user information
echo "create temp json file [${tempResourceJsonFile}]"
echo '{ "name": "'${tempResourceName}'", "description": "Top level resource" }' > ${tempResourceJsonFile}
sleep 1
echo "dumping contents of [${tempResourceJsonFile}]"
cat ${tempResourceJsonFile}
# create the new temporary resource
echo Creating temporary resource
createResource ${tempResourceJsonFile}
# add resource to team
addResourceToTeam "/"${tempResourceName} "${QDteam}"
# delete temporary resource file
echo "delete the temporary resource file"
deleteFile ${workingDir} ${tempResourceJsonFileName}

# Create default environment
echo Creating default application environment

createEnvironment ${appName} "default-environment"
addEnvironmentToTeam ${appName} "default-environment" "${QDteam}"

# -- stage 8: Creating default blueprints --
# ------------------------------------------
echo "==================================================================="
echo "=========== STAGE 8 : create QD environment blueprints ============"
echo "==================================================================="
currentStage="stage 8"
# Create default blueprints
echo Creating default blueprints
createDefaultBlueprints ${appName}

# -- the end:
echo "==================================================================="
errMsg="reached the end of the script"
exitWithOkMsg "$errMsg"
